@@ -16,7 +16,6 @@
 /* Defined when your system has a define or typedef for sig_t. */
 #undef HAVE_TYPE_SIG_T
 
-
 /* Defined when your system has a define or typedef for time_t. */
 #undef HAVE_TYPE_TIME_T
 
@@ -35,4 +34,7 @@
 #  define METAHTML_PROFILER 1
 #endif
 
+#if defined (HAVE_LIBCRYPT) && (!defined (HAVE_CRYPT))
+#  define HAVE_CRYPT 1
+#endif
 
