@@ -37,7 +37,7 @@
 	   ;;; (modify-syntax-entry ?\) "-" mhtml-mode-syntax-table)
 	   (modify-syntax-entry ?\; "w" mhtml-mode-syntax-table)
 	   (modify-syntax-entry ?\& "w" mhtml-mode-syntax-table)
-	   (modify-syntax-entry ?\| "w" mhtml-mode-syntax-table)))
+	   (modify-syntax-entry ?\| "." mhtml-mode-syntax-table)))
 
 (defun mhtml-mode-variables (mhtml-syntax)
   (set-syntax-table mhtml-mode-syntax-table)
@@ -60,7 +60,7 @@
   (make-local-variable 'comment-start)
   (setq comment-start ";;; ")
   (make-local-variable 'comment-start-skip)
-  (setq comment-start-skip ";;+ *")
+  (setq comment-start-skip ";;;+ *")
   (make-local-variable 'comment-column)
   (setq comment-column 40)
   (make-local-variable 'comment-indent-function)
@@ -75,6 +75,7 @@
     (define-key map [comment-region] '("Comment Out Region" . comment-region))
     (define-key map [indent-region] '("Indent Region" . indent-region))
     (define-key map "\e\C-q" 'mhtml-indent-region)
+    (define-key map "\eq" 'mhtml-indent-sexp)
     (define-key map [indent-line] '("Indent Line" . indent-relative))
     (put 'eval-region 'menu-enable 'mark-active)
     (put 'comment-region 'menu-enable 'mark-active)
