@@ -40,6 +40,12 @@
 #include <termios.h>
 #endif
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
+
 #if defined (BUILDING_WITH_FAST_CGI)
 #  include "./fcgi/include/fcgi_stdio.h"
 #endif
@@ -89,7 +95,7 @@ static void fatal (char *format, ...);
 
 #if defined (CHECK_ACTIVATION)
 extern int check_activation_key (char *key);
-#endif
+#endif /* CHECK_ACTIVATION */
 
 static void process_request (void);
 static void find_config_file (void);
@@ -394,22 +400,23 @@ file_exists_p (char *path)
   return (stat (path, &finfo) != -1);
 }
 
+
 static char *
-find_config_locally (char *dir)
-{
+find_config_in_dir (char *dir, char *filename)
+{  
   char *result = (char *)NULL;
-  char path[1024 + 256];
+  char path[1024 + 512];
   char *temp;
 
   /* Try ./engine.conf */
-  sprintf (path, "%s/engine.conf", dir);
+  sprintf (path, "%s/%s", dir, filename);
 
   if (file_exists_p (path))
     result = strdup (path);
   else
     {
       /* Try ./conf/engine.conf */
-      sprintf (path, "%s/conf/engine.conf", dir);
+      sprintf (path, "%s/conf/%s", dir, filename);
       if (file_exists_p (path))
 	result = strdup (path);
       else
@@ -420,14 +427,13 @@ find_config_locally (char *dir)
 	  if (temp)
 	    {
 	      *temp = '\0';
-	      strcat (path, "/engine.conf");
+	      sprintf (temp, "/%s", filename);
 	      if (file_exists_p (path))
 		result = strdup (path);
 	      else
 		{
 		  /* Try ../conf/engine.conf */
-		  *temp = '\0';
-		  strcat (path, "/conf/engine.conf");
+		  sprintf (temp, "/conf/%s", filename);
 		  if (file_exists_p (path))
 		    result = strdup (path);
 		}
@@ -438,6 +444,32 @@ find_config_locally (char *dir)
   return (result);
 }
 
+static char *
+find_config_locally (char *dir)
+{
+  char *result = (char *)NULL;
+  char filename[512];
+  char *hostname = (char *)getenv ("HTTP_HOST");
+
+  /* If there is a specific host config file, use that first. */
+  if ((hostname != (char *)NULL) && (hostname[0] != '\0'))
+    {
+      sprintf (filename, "%s.conf", hostname);
+      result = find_config_in_dir (dir, filename);
+
+      if (result == (char *)NULL)
+	{
+	  sprintf (filename, "%s-engine.conf", hostname);
+	  result = find_config_in_dir (dir, filename);
+	}
+    }
+
+  if (result == (char *)NULL)
+    result = find_config_in_dir (dir, "engine.conf");
+
+  return (result);
+}
+
 static void
 find_config_file (void)
 {
@@ -810,3 +842,7 @@ process_request (void)
       free (result);
     }
 }
+
+#if defined (__cplusplus)
+}
+#endif
