@@ -30,6 +30,11 @@
 
 #include "language.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 static void pf_array_size (PFunArgs);
 static void pf_array_member (PFunArgs);
 static void pf_array_append (PFunArgs);
@@ -226,12 +231,18 @@ See also <funref arrays array-append>.")
   xfree (item);
 }
 
-DEFUN (pf_array_shift, amount arrayvar,
+DEFUN (pf_array_shift, amount arrayvar &key start,
 "Shift the elements of <var arrayvar> the indicated amount.
 
-If <var amount> is negative, the elements are shifted down (i.e. towards zero), with the lowest number elements being lost.
+If <var amount> is negative, the elements are shifted down
+(i.e. towards zero), with the lowest number elements being lost.
 
-If <var amount> is positive, the elements are shifted up, with no loss at all -- instead empty elements are used to fill the created space.
+If <var amount> is positive, the elements are shifted up, with
+no loss at all -- instead empty elements are used to fill the
+created space.
+
+If the keyword argument <var start> is present, it indicates the
+zero-based offset from which to start shifting.
 
 Given the array:
 <example>
@@ -248,7 +259,8 @@ Given the array:
        1
        2>>
 
-then after executing <example code><array-shift 2 array></example>, the array looks like:
+then after executing <example code><array-shift 2 array></example>, 
+the array looks like:
 <example>
    \"\"
    \"\"
@@ -256,7 +268,8 @@ then after executing <example code><array-shift 2 array></example>, the array lo
    \"1\"
    \"2\"
 </example>
-and, a subsequent execution of <example code><array-shift -3 array></example> leaves <var array>:
+and, a subsequent execution of
+<example code><array-shift -3 array></example> leaves <var array>:
 <example>
    \"1\"
    \"2\"
@@ -424,6 +437,10 @@ or, we can produce a \"countdown\" with a negative value for <var step>:
 	  if (((step < 0) && (end_index < start_index)) ||
 	      ((step > 0) && (end_index > start_index)))
 	    {
+	      /* Now copy the array, so that functions which delete or
+		 manipulate the contents of it do not disturb our pointers. */
+	      char **array_values = symbol_copy_array (array->values);
+
 	      for (i = start_index;
 		   (((start_index < end_index) && (i < end_index)) ||
 		    ((start_index > end_index) && (i >= end_index)));
@@ -435,7 +452,7 @@ or, we can produce a \"countdown\" with a negative value for <var step>:
 
 		  symbol_free (element);
 		  element = symbol_intern (element_var);
-		  symbol_add_value (element, array->values[i]);
+		  symbol_add_value (element, array_values[i]);
 		  code = page_copy_page (body);
 		  page_process_page_internal (code);
 		  parser_current_lineno = line;
@@ -454,6 +471,8 @@ or, we can produce a \"countdown\" with a negative value for <var step>:
 		      if (broken) break;
 		    }
 		}
+
+	      symbol_free_array (array_values);
 	    }
 
 	  *newstart = start;
@@ -740,3 +759,7 @@ which calculates that value for each string:
 
   if (sortvar) free (sortvar);
 }
+
+#if defined (__cplusplus)
+}
+#endif
