@@ -6,6 +6,11 @@
 #include "language.h"
 #include "symdump.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 extern unsigned char bootstrap_code[];
 extern int bootstrap_code_len;
 
@@ -43,3 +48,7 @@ bootstrap_metahtml (int call_initializer_p)
 	}
     }
 }
+
+#if defined (__cplusplus)
+}
+#endif
