@@ -31,6 +31,11 @@
 #define NEW_CGI_EXEC 1
 #include "language.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 #if defined (NEW_CGI_EXEC)
 #  if defined (__WINNT__)
      /* Needed for select(). */
@@ -460,3 +465,7 @@ in <funref language-operators SYSTEM-ERROR-OUTPUT>.")
       xfree (errors_text);
     }
 }
+
+#if defined (__cplusplus)
+}
+#endif
