@@ -31,6 +31,10 @@
 #if !defined (_DATABASE_H_)
 #define _DATABASE_H_
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 /* An opaque handle to a database. */
 typedef void *DBFILE;
 
@@ -81,5 +85,8 @@ extern int database_had_error_p (void);
 /* Reset the database error indicator. */
 extern void database_reset_error (void);
 
-#endif /* !_DATABASE_H_ */
+#if defined (__cplusplus)
+}
+#endif
 
+#endif /* !_DATABASE_H_ */
