@@ -30,6 +30,11 @@
 
 #include "language.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 static void pf_with_open_database (PFunArgs);
 static void pf_database_load_record (PFunArgs);
 static void pf_database_delete_record (PFunArgs);
@@ -172,8 +177,22 @@ unlock_database (char *dbname, int *lock, DBFILE *db)
   if (*lock > -1)
     {
       char *lockname = db_lockname (dbname);
-      unlink (lockname); 
+      char *unlockname = (char *)NULL;
+
+      if (lockname != (char *)NULL)
+	{
+	  unlockname = (char *)xmalloc (10 + strlen (lockname));
+	  strcpy (unlockname, lockname);
+	  strcat (unlockname, ".unlock");
+	  rename (lockname, unlockname);
+	}
+
       UNLOCKFILE (*lock);
+      if (unlockname != (char *)NULL)
+	{
+	  unlink (unlockname);
+	  free (unlockname);
+	}
       close (*lock);
     }
 }
@@ -304,7 +323,7 @@ dbkey_function_args (Package *vars, DBFILE *db, DBOBJ **key, int key_required)
 	}
     }
 
-  if (dbref) free (dbref);
+  xfree (dbref);
   if (dbkey && !key) free (dbkey);
 }
 
@@ -568,9 +587,9 @@ written.
 Upon success, this function returns the string <code>true</code>.
 
 <example>
-<set-var foo::x=x foo::array[0]=val0 foo::array[1]=val1>
+<set-var mykey="The Key" foo::x=x foo::array[0]=val0 foo::array[1]=val1>
 <with-open-database db /tmp/file.db mode=write-create>
-  <database-save-package db mykey foo strip=true>
+  <database-save-package db <get-var mykey> foo strip=true>
 </with-open-database>
 </example>
 
@@ -588,14 +607,14 @@ true
   if (db && key)
     {
       int strip = !empty_string_p (get_value (vars, "strip"));
-      char *package_name = mhtml_evaluate_string (get_positional_arg (vars, 2));
+      char *package_arg = mhtml_evaluate_string (get_positional_arg (vars, 2));
       Symbol **symbols = (Symbol **)NULL;
 
-      if (!empty_string_p (package_name))
-	symbols = symbol_package_symbols (package_name);
+      if (!empty_string_p (package_arg))
+	symbols = symbol_package_symbols (package_arg);
 
-      if (package_name != (char *)NULL)
-	free (package_name);
+      if (package_arg != (char *)NULL)
+	free (package_arg);
 
       if (symbols != (Symbol **)NULL)
 	{
@@ -792,7 +811,7 @@ modifications to the suggestion.")
 /* Read the record in DB at KEY, and return a package containing the
    fields and values. */
 static Package *
-database_package_contents (DBFILE *db, DBOBJ *key)
+database_package_contents (DBFILE db, DBOBJ *key)
 {
   DBOBJ *content = database_fetch (db, key);
   Package *package = (Package *)NULL;
@@ -1335,3 +1354,7 @@ between the ages of 20 and 45, by creating links to a page which will
   if (dbref) free (dbref);
   if (sort_args) free (sort_args);
 }
+
+#if defined (__cplusplus)
+}
+#endif
