@@ -31,6 +31,10 @@
 
 #include "language.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 /************************************************************/
 /*							    */
 /*		   Debugger Facilitation Functions	    */
@@ -223,3 +227,7 @@ standalone script.")
       *newstart = start + strlen (token_name) - 1;
     }
 }
+
+#if defined (__cplusplus)
+}
+#endif
