@@ -32,6 +32,11 @@
 
 #include "language.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 static void pf_with_open_database (PFunArgs);
 static void pf_database_query (PFunArgs);
 static void pf_database_exec_query (PFunArgs);
@@ -189,6 +194,52 @@ istack_free (IStack *stack, FREEFUN *free_fun)
   free (stack);
 }
 
+/* Look up a key-value pair in a ODBC style DSN string.
+
+   Example:
+       HOST=star;SVT=Informix 5;UID=demo;PWD=demo;DATABASE=stores5 */
+static char *
+dsn_lookup (char *key, char *str)
+{
+  char *dsn = strdup (str);
+  char *kp = dsn;
+  char *vp = (char *)NULL;
+  char *val = (char *)NULL;
+  int found = 0;
+
+  while ((vp = strchr (kp, '=')) != (char *) NULL)
+    {
+      char *semi;
+
+      *vp = 0;
+      vp++;
+
+      /* Trim off semicolon separator, if there is one */
+      semi = strchr (vp, ';');
+      if (semi != (char *)NULL)
+	*semi = 0;
+
+      if (strcasecmp (kp, key) == 0)
+	{
+	  found = 1;
+	  break;
+	}
+
+      if (semi == (char *)NULL)
+	break;
+      else
+	kp = ++semi;
+    }
+
+  if (found == 1)
+    val = strdup (vp);
+  else
+    val = (char *)NULL;
+
+  free (dsn);
+  return (val);
+}
+
 #if defined (COMPILING_MSQLFUNCS)
 #  include "msqlfuncs.c"
 #endif
@@ -378,7 +429,7 @@ get_cursor_ref (Package *vars)
 			 (gsql_field_type (field) == GSQL_LONG_BLOB) || \
 			 (gsql_field_type (field) == GSQL_BLOB))
 
-/* We pass an string which contains a numeric representation of the
+/* We pass an string which contains a decimal representation of the
    SQL escape code character, such as "39" */
 static void
 gsql_database_set_sql_escape_character (Database *db, char *escape_code)
@@ -496,17 +547,30 @@ lock_database (char *dsn, int *lock, Database *db, int lock_p)
 static void
 unlock_database (char *dsn, int *lock, Database *db, int lock_p)
 {
-  char *lockname = db_lockname (dsn);
-
   if (db != NULL)
     gsql_close (db);
 
   if (lock_p && (*lock > -1))
     {
+      char *lockname = db_lockname (dsn);
+      char *unlockname = (char *)NULL;
+
       if (lockname != (char *)NULL)
-	unlink (lockname);
+	{
+	  unlockname = (char *)xmalloc (10 + strlen (lockname));
+	  strcpy (unlockname, lockname);
+	  strcat (unlockname, ".unlock");
+	  rename (lockname, unlockname);
+	}
 
       UNLOCKFILE (*lock);
+
+      if (unlockname != (char *)NULL)
+	{
+	  unlink (unlockname);
+	  free (unlockname);
+	}
+
       close (*lock);
     }
 }
@@ -579,7 +643,9 @@ pf_with_open_database (PFunArgs)
   if ((empty_string_p (dsn)) && (!empty_string_p (dbname)))
     {
       BPRINTF_BUFFER *service = bprintf_create_buffer ();
+
       bprintf (service, "HOST=%s;DATABASE=%s", hostname, dbname);
+      xfree (dsn);
       dsn = strdup (service->buffer);
       bprintf_free_buffer (service);
     }
@@ -876,6 +942,7 @@ pf_cursor_get_column (PFunArgs)
   if ((database_environment_level != 0) && !(empty_string_p (colnum)))
     {
       DBCursor *cursor = get_cursor_ref (vars);
+
       if (cursor != (DBCursor *) NULL)
 	{
 	  gsql_result *query_result = cursor->result;
@@ -1562,7 +1629,8 @@ pf_database_query (PFunArgs)
       register int i;
       char *query = mhtml_evaluate_string (get_value (vars, "QUERY"));
       char *keys_var = mhtml_evaluate_string (get_value (vars, "KEYS"));
-      char *keyname = mhtml_evaluate_string (get_value (vars, "KEYNAME"));
+      char *keyname = mhtml_evaluate_string
+	(get_one_of (vars, "KEYNAME", "KEY", (char **)NULL));
       char *colnames = mhtml_evaluate_string (get_value (vars, "COLNAMES"));
       char *prefixtablenames =
 	mhtml_evaluate_string (get_value (vars, "PrefixTableNames"));
@@ -1876,8 +1944,8 @@ bprintf_gsql_escape (BPRINTF_BUFFER *out, char *src, gsql_field *field,
       i++;
       if ((maxlength != 0) && (i > maxlength)) return;
 
-      /* Oracle doesn't need parens quoted, but MSQL does */
-#if defined (COMPILING_MSQLFUNCS)
+      /* Oracle doesn't need parens quoted, but mSQL and MySQL do. */
+#if defined (COMPILING_MSQLFUNCS) || defined (COMPILING_MYSQLFUNCS)
       if ((c == '\'') || (c == '(') || (c == ')'))
 	bprintf (out, "%s", escape_string);
 #else
@@ -1959,7 +2027,8 @@ pf_database_save_record (PFunArgs)
 {
   Database *db = get_dbref (vars);
   char *key     = mhtml_evaluate_string (get_positional_arg (vars, 1));
-  char *keyname = mhtml_evaluate_string (get_value (vars, "keyname"));
+  char *keyname = mhtml_evaluate_string
+    (get_one_of (vars, "keyname", "key", (char *)NULL));
   char *table   = mhtml_evaluate_string (get_value (vars, "table"));
   char *method   = mhtml_evaluate_string (get_value (vars, "method"));
   BPRINTF_BUFFER *query = (BPRINTF_BUFFER *) NULL;
@@ -2192,7 +2261,8 @@ pf_database_load_record (PFunArgs)
 {
   Database *db = get_dbref (vars);
   char *key     = mhtml_evaluate_string (get_positional_arg (vars, 1));
-  char *keyname = mhtml_evaluate_string (get_value (vars, "keyname"));
+  char *keyname = mhtml_evaluate_string
+    (get_one_of (vars, "keyname", "key", (char *)NULL));
   char *table   = mhtml_evaluate_string (get_value (vars, "table"));
   BPRINTF_BUFFER *query = (BPRINTF_BUFFER *) NULL;
   char *pkgname = (char *) NULL;
@@ -2326,7 +2396,8 @@ pf_database_delete_record (PFunArgs)
 {
   Database *db = get_dbref (vars);
   char *key     = mhtml_evaluate_string (get_positional_arg (vars, 1));
-  char *keyname = mhtml_evaluate_string (get_value (vars, "keyname"));
+  char *keyname = mhtml_evaluate_string
+    (get_one_of (vars, "keyname", "key", (char *)NULL));
   char *table   = mhtml_evaluate_string (get_value (vars, "table"));
   BPRINTF_BUFFER *query = (BPRINTF_BUFFER *) NULL;
   char *result = (char *)NULL;
@@ -2417,7 +2488,7 @@ pf_database_save_package (PFunArgs)
 {
   Database *db = get_dbref (vars);
   char *key     = mhtml_evaluate_string (get_positional_arg (vars, 1));
-  char *keyname = mhtml_evaluate_string (get_value (vars, "keyname"));
+  char *keyname = mhtml_evaluate_string (get_one_of (vars, "keyname", "key", (char *)NULL));
   char *table   = mhtml_evaluate_string (get_value (vars, "table"));
   char *method   = mhtml_evaluate_string (get_value (vars, "method"));
   BPRINTF_BUFFER *query = (BPRINTF_BUFFER *) NULL;
@@ -2477,7 +2548,6 @@ pf_database_save_package (PFunArgs)
 	    }
 	}
 
-
       {
 	char *packname = mhtml_evaluate_string (get_positional_arg (vars, 2));
 
@@ -2728,9 +2798,9 @@ pretty_field_type (int fieldtype)
 }
 
 static char *
-string_from_boolean (int bool)
+string_from_boolean (int the_bool)
 {
-  if (bool == 0) return "";
+  if (the_bool == 0) return "";
   else return "true";
 }
 
@@ -2757,7 +2827,7 @@ alist_of_field_info (gsql_field *field, Database *db)
 	       string_from_boolean (! (gsql_field_is_not_null (field))));
       bprintf (tmpbuf, "(QUALIFIER . \"%s\")", gsql_field_qualifier (field));
       bprintf (tmpbuf, "(OWNER . \"%s\")", gsql_field_owner (field));
-      bprintf (tmpbuf, "(TYPENAME . \"%s\")", gsql_field_typename (field));
+      bprintf (tmpbuf, "(TYPENAME . \"%s\")", gsql_field_type_name (field));
       bprintf (tmpbuf, "(PRECISION . \"%d\")", gsql_field_precision (field));
       bprintf (tmpbuf, "(SCALE . \"%d\")", gsql_field_scale (field));
       bprintf (tmpbuf, "(RADIX . \"%d\")", gsql_field_radix (field));
@@ -3100,3 +3170,7 @@ pf_database_tables_info (PFunArgs)
   xfree (name);
   xfree (type);
 }
+
+#if defined (__cplusplus)
+}
+#endif
