@@ -30,10 +30,17 @@
 
 #include "language.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 #if defined (MHTML_ARITHMETIC)
+
 static void pf_gt (PFunArgs);
 static void pf_lt (PFunArgs);
 static void pf_eq (PFunArgs);
+static void pf_neq (PFunArgs);
 static void pf_add (PFunArgs);
 static void pf_sub (PFunArgs);
 static void pf_mul (PFunArgs);
@@ -45,6 +52,7 @@ static PFunDesc func_table[] =
   { "GT",		0, 0, pf_gt },
   { "LT",		0, 0, pf_lt },
   { "EQ",		0, 0, pf_eq },
+  { "NEQ",		0, 0, pf_neq },
   { "ADD",		0, 0, pf_add },
   { "SUB",		0, 0, pf_sub },
   { "MUL",		0, 0, pf_mul },
@@ -60,6 +68,7 @@ PACKAGE_INITIALIZER (initialize_arithmetic_functions)
 /* <gt  12 10> --> "true"
    <lt  10 12> --> "true"
    <eq  10 10> --> "true"
+   <neq 10 10> --> "false"
    <add 10 10> --> "20"
    <sub 10 10> --> "0"
    <mul 10 10> --> "100"
@@ -68,17 +77,19 @@ PACKAGE_INITIALIZER (initialize_arithmetic_functions)
 #define pf_GT	1
 #define pf_LT	2
 #define pf_EQ	3
-#define pf_ADD	4
-#define pf_SUB	5
-#define pf_MUL	6
-#define pf_DIV	7
-#define pf_MOD	8
+#define pf_NEQ	4
+#define pf_ADD	5
+#define pf_SUB	6
+#define pf_MUL	7
+#define pf_DIV	8
+#define pf_MOD	9
 
 typedef struct { int op; char *name; } OP_ALIST;
 static OP_ALIST op_alist[] = {
   { pf_GT, "GT" },
   { pf_LT, "LT" },
   { pf_EQ, "EQ" },
+  { pf_EQ, "NEQ" },
   { pf_ADD, "ADD" },
   { pf_SUB, "SUB" },
   { pf_MUL, "MUL" },
@@ -127,18 +138,35 @@ arithmetic_operate (int op, char *arg1, char *arg2)
       break;
 
     case pf_EQ:
+    case pf_NEQ:
       if (!arg1) arg1 = "";
       if (!arg2) arg2 = "";
 
       if (number_p (arg1) && number_p (arg2))
 	{
-	  if (val1 == val2)
-	    result = "true";
+	  if (op == pf_EQ)
+	    {
+	      if (val1 == val2)
+		result = "true";
+	    }
+	  else if (op == pf_NEQ)
+	    {
+	      if (val1 != val2)
+		result = "true";
+	    }
 	}
       else
 	{
-	  if (strcasecmp (arg1, arg2) == 0)
-	    result = "true";
+	  if (op == pf_EQ)
+	    {
+	      if (strcasecmp (arg1, arg2) == 0)
+		result = "true";
+	    }
+	  else if (op == pf_NEQ)
+	    {
+	      if (strcasecmp (arg1, arg2) != 0)
+		result = "true";
+	    }
 	}
       break;
 
@@ -200,7 +228,8 @@ arithmetic_operate (int op, char *arg1, char *arg2)
   return (result);
 }
 
-#define relational_op(op) ((op == pf_GT) || (op == pf_LT) || (op == pf_EQ))
+#define relational_op(op) \
+	((op == pf_GT) || (op == pf_LT) || (op == pf_EQ) || (op == pf_NEQ))
 
 static void
 arithmetic_process (int op, PFunArgs)
@@ -242,8 +271,9 @@ arithmetic_process (int op, PFunArgs)
 	  if (strcmp (arg, temp) == 0)
 	    {
 	      /* No, so try looking it up as a variable name.
-		 Only do this when the operation is not pf_EQ. */
-	      /* if (op != pf_EQ) */
+		 Only do this when the operation is not pf_EQ
+		 or pf_NEQ. */
+	      /* if ((op != pf_EQ) && (op != pf_NEQ)) */
 		{
 		  for (i = 0; whitespace (arg[i]); i++);
 		  value_pointer = pagefunc_get_variable (arg + i);
@@ -391,6 +421,27 @@ Examples:
   arithmetic_process (pf_EQ, PassPFunArgs);
 }
 
+DEFUN (pf_neq, arg1 arg2,
+"Returns \"true\" if the numeric value of <var arg1> is NOT equal to
+the numeric value of <var arg2>.  Just as with the arithmetic
+functions (<secref Arithmetic-Operators>), the arguments
+may be the names of variables containing numeric values, and not just
+the values themselves.  In that case, <code>eq</code> performs an
+implicit call to <example code><get-var ...></example> in order to get
+the value to operate on.
+
+Examples:
+<example>
+<neq 3 4>              --> true
+<neq 3 3>              --> 
+<neq 3.0 3>            --> 
+<set-var x=3.01>
+<neq <get-var x> 3.00> --> true
+</example>")
+{
+  arithmetic_process (pf_NEQ, PassPFunArgs);
+}
+
 DOC_SECTION (ARITHMETIC-OPERATORS)
 
 DEFUN (pf_add, arg1 arg2 &rest more-args,
@@ -481,3 +532,7 @@ void initialize_arithmetic_functions (Package *package) {}
 
 #endif /* MHTML_ARITHMETIC */
 
+#if defined (__cplusplus)
+}
+#endif
+
