@@ -112,7 +112,11 @@
 #include <signal.h>
 
 #include <setjmp.h>
-#include <regex/regex.h>
+#if !defined (HAVE_REGEX_H)
+#  include <regex/regex.h>
+#else
+#  include <regex.h>
+#endif
 #include <bprintf/bprintf.h>
 #include <xmalloc/xmalloc.h>
 #include <tcp/tcp.h>
@@ -123,8 +127,12 @@
 #endif
 
 #if !defined (HAVE_TYPE_SIG_T)
+#if defined (__cplusplus)
+extern "C" typedef RETSIGTYPE (*sig_t) (int);
+#else
 typedef RETSIGTYPE (*sig_t) (int);
-#endif
+#endif /* CPLUSPLUS */
+#endif /* HAVE_TYPE_SIG_T */
 
 #if !defined (HAVE_TYPE_TIME_T)
 typedef long time_t;
