@@ -65,6 +65,53 @@ static PFunDesc func_table[] =
 };
 
 PACKAGE_INITIALIZER (initialize_mysql_functions)
+DOC_SECTION (MYSQL-DATABASE-EXTENSIONS)
+DEFMACROX (pf_mysql::with_open_database, ,
+"See <funref generic-sql-interface sql::with-open-database>.")
+DEFUNX (pf_mysql::database_exec_query, ,
+"See <funref generic-sql-interface sql::database-exec-query>.")
+DEFUNX (pf_mysql::database_exec_sql, ,
+"See <funref generic-sql-interface sql::database-exec-sql>.")
+DEFUNX (pf_mysql::database_next_record, ,
+"See <funref generic-sql-interface sql::database-next-record>.")
+DEFUNX (pf_mysql::database_save_record, ,
+"See <funref generic-sql-interface sql::database-save-record>.")
+DEFUNX (pf_mysql::database_delete_record, ,
+"See <funref generic-sql-interface sql::database-delete-record>.")
+DEFUNX (pf_mysql::database_load_record, ,
+"See <funref generic-sql-interface sql::database-load-record>.")
+DEFUNX (pf_mysql::database_save_package, ,
+"See <funref generic-sql-interface sql::database-save-package>.")
+DEFUNX (pf_mysql::database_num_rows, ,
+"See <funref generic-sql-interface sql::database-num-rows>.")
+DEFUNX (pf_mysql::database_affected_rows, ,
+"See <funref generic-sql-interface sql::database-affected-rows>.")
+DEFUNX (pf_mysql::database_set_pos, ,
+"See <funref generic-sql-interface sql::database-set-pos>.")
+DEFUNX (pf_mysql::database_query, ,
+"See <funref generic-sql-interface sql::database-query>.")
+DEFUNX (pf_mysql::host_databases, ,
+"See <funref generic-sql-interface sql::host-databases>.")
+DEFUNX (pf_mysql::database_tables, ,
+"See <funref generic-sql-interface sql::database-tables>.")
+DEFUNX (pf_mysql::database_tables_info, ,
+"See <funref generic-sql-interface sql::database-tables-info>.")
+DEFUNX (pf_mysql::database_columns, ,
+"See <funref generic-sql-interface sql::database-columns>.")
+DEFUNX (pf_mysql::database_column_info, ,
+"See <funref generic-sql-interface sql::database-column-info>.")
+DEFUNX (pf_mysql::database_columns_info, ,
+"See <funref generic-sql-interface sql::database-columns-info>.")
+DEFUNX (pf_mysql::database_query_info, ,
+"See <funref generic-sql-interface sql::database-query-info>.")
+DEFUNX (pf_mysql::database_set_options, ,
+"See <funref generic-sql-interface sql::database-set-options>.")
+DEFUNX (pf_mysql::cursor_get_column, ,
+"See <funref generic-sql-interface sql::cursor-get-column>.")
+DEFUNX (pf_mysql::query_get_column, ,
+"See <funref generic-sql-interface sql::query-get-column>.")
+DEFUNX (pf_mysql::sql_transact, ,
+"See <funref generic-sql-interface sql::sql-transact>.")
 
 /****************************************************************
  * The Database object:
@@ -73,52 +120,6 @@ PACKAGE_INITIALIZER (initialize_mysql_functions)
  * database connection.
  ****************************************************************/
 
-/* Look up a key-value pair in a ODBC style DSN string.
-
-   Example:
-       HOST=star;SVT=Informix 5;UID=demo;PWD=demo;DATABASE=stores5 */
-static char *
-dsn_lookup (char *key, char *str)
-{
-  char *dsn = strdup (str);
-  char *kp = dsn;
-  char *vp = (char *)NULL;
-  char *val = (char *)NULL;
-  int found = 0;
-
-  while ((vp = strchr (kp, '=')) != (char *) NULL)
-    {
-      char *semi;
-
-      *vp = 0;
-      vp++;
-
-      /* Trim off semicolon separator, if there is one */
-      semi = strchr (vp, ';');
-      if (semi != (char *)NULL)
-	*semi = 0;
-
-      if (strcasecmp (kp, key) == 0)
-	{
-	  found = 1;
-	  break;
-	}
-
-      if (semi == (char *)NULL)
-	break;
-      else
-	kp = ++semi;
-    }
-
-  if (found == 1)
-    val = strdup (vp);
-  else
-    val = (char *)NULL;
-
-  free (dsn);
-  return (val);
-}
-
 typedef struct
 {
   IStack *cursors;
@@ -141,7 +142,7 @@ typedef struct
      future compatibility with ODBC. */
   char *qualifier;
   char *owner;
-  char *typename;
+  char *type_name;
   int precision;
   int scale;
   int radix;
@@ -336,7 +337,7 @@ initialize_gsql_field (gsql_field *gfield)
 {
   gfield->qualifier = (char *) NULL;
   gfield->owner     = (char *) NULL;
-  gfield->typename  = (char *) NULL;
+  gfield->type_name  = (char *) NULL;
   gfield->precision = 0;
   gfield->scale     = 0;
   gfield->radix     = 0;
@@ -358,7 +359,8 @@ gsql_make_field_array (MYSQL_RES *result)
 
   if (numfields > 0)
     {
-      gr->fields = xmalloc ((numfields + 1) * sizeof (gsql_field *));
+      gr->fields = (gsql_field **) xmalloc
+	((numfields + 1) * sizeof (gsql_field *));
 
       for (i = 0; i < numfields; i++)
 	{
@@ -432,7 +434,7 @@ gsql_db_list_tables (Database *db,
 /* Future ODBC compatibility */
 #define gsql_field_qualifier(f) (f->qualifier)
 #define gsql_field_owner(f) (f->owner)
-#define gsql_field_typename(f) (f->typename)
+#define gsql_field_type_name(f) (f->type_name)
 #define gsql_field_precision(f) (f->precision)
 #define gsql_field_scale(f) (f->scale)
 #define gsql_field_radix(f) (f->radix)
@@ -493,7 +495,7 @@ gsql_num_fields (gsql_result *result)
 static char *
 gsql_get_column (gsql_result *result, int col)
 {
-  if (result->mysql_row[col])
+  if ((result->mysql_row != (MYSQL_ROW) NULL) && (result->mysql_row[col]))
     return (strdup (result->mysql_row[col]));
   else
     return ((char *)NULL);
@@ -577,9 +579,9 @@ gsql_close (Database *db)
     }
 }
 
-/* <mysql::host-databases [hostname] [result=varname]>
-   Returns an array of the databases available on HOST.
-   If VARNAME is supplied, the array is placed into that variable instead. */
+/* Returns an array of the databases available on HOST.
+   If VARNAME is supplied, the array is placed into that
+   variable instead. */
 static void
 pf_host_databases (PFunArgs)
 {
