@@ -30,6 +30,10 @@
 
 #include "language.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 /************************************************************/
 /*							    */
 /*		Language Manipulation Functions		    */
@@ -38,6 +42,7 @@
 
 static void pf_randomize (PFunArgs);
 static void pf_random (PFunArgs);
+static void pf_make_identifier (PFunArgs);
 
 /* <date> --> Mon Feb 12 04:28:15 PST 1996 */
 static void pf_date (PFunArgs);
@@ -115,6 +120,7 @@ static PFunDesc func_table[] =
 
   { "RANDOMIZE",	0, 0, pf_randomize },
   { "RANDOM",		0, 0, pf_random },
+  { "MAKE-IDENTIFIER",	0, 0, pf_make_identifier },
 
   { "DATE",		0, 0, pf_date },
   { "TIME",		0, 0, pf_time },
@@ -195,6 +201,52 @@ Also see <funref arithmetic-operators randomize>.")
 
 DOC_SECTION (LANGUAGE-OPERATORS)
 
+DEFUN (pf_make_identifier, limit,
+"Create an identifier that is <var limit> characters in length.
+The identifier characters are taken from the string MI::ALPHABET.
+MI::ALPHABET defaults to uppercase letters and numbers excluding
+\"I\", \"O\", zero, and one.")
+{
+  register int i;
+  char *limit_arg = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  char *alphabet = pagefunc_get_variable ("MI::ALPHABET");
+  int alphabet_length, limit = 0;
+  static char *default_alphabet = "ABCDEFGHJK23456789MNPQRSTYVWXYZ98765432";
+  char *identifier = (char *)NULL;
+
+  if (empty_string_p (alphabet))
+    alphabet = default_alphabet;
+
+  alphabet_length = strlen (alphabet);
+
+  if (!empty_string_p (limit_arg))
+    limit = atoi (limit_arg);
+
+  xfree (limit_arg);
+
+  if (!limit) limit = 16;
+
+  identifier = (char *)xmalloc (1 + limit);
+
+  if (!randomize_called)
+    {
+      srandom ((unsigned int)getpid ());
+      randomize_called++;
+    }
+
+  for (i = 0; i < limit; i++)
+    {
+      int offset = (random () % alphabet_length);
+      identifier[i] = alphabet[offset];
+    }
+
+  identifier[i] = '\0';
+
+  bprintf_insert (page, start, "%s", identifier);
+  *newstart += strlen (identifier);
+  free (identifier);
+}
+
 DEFMACRO (pf_comment, ,
 "Simply discard the <var body>.  No processing is done, and no output
 is produced.
@@ -280,12 +332,14 @@ value is given, then that value is placed into the page instead.")
 	    }
 	  else
 	    {
+	      char digits[40];
 	      found = 1;
+	      sprintf (digits, "%ld", page_most_recent_modification_time);
+	      pagefunc_set_variable ("mhtml::last-modification-time", digits);
 	    }
 
 	  if (file_contents)
 	    {
-
 	      if (found && !verbatim_p)
 		{
 		  bprintf_insert (file_contents, 0, "<*parser*::push-file %s>",
@@ -513,10 +567,10 @@ html_quote_string (char *string)
 
 DOC_SECTION (PAGE-OPERATORS)
 
-DEFUN (pf_subst_in_page, this with-that,
-"Replaces all occurrences of <var this> with <var with-that> in the
-current page.  Both <var this> and <var with-that> are evaluated
-before the replacement is done.  <var this> can be any regular
+DEFUN (pf_subst_in_page, this-string with-that,
+"Replaces all occurrences of <var this-string> with <var with-that> in the
+current page.  Both <var this-string> and <var with-that> are evaluated
+before the replacement is done.  <var this-string> can be any regular
 expression allowed by the POSIX extended regular expression matching.
 
 This command can be useful when included as part of a footer
@@ -532,24 +586,24 @@ occurrences of <code>%name%</code> with a the value of the variable
 
   while (!done)
     {
-      char *this, *that;
+      char *this_string, *with_that;
 
-      this = get_positional_arg (vars, arg++);
-      that = get_positional_arg (vars, arg++);
+      this_string = get_positional_arg (vars, arg++);
+      with_that = get_positional_arg (vars, arg++);
 
-      if (this == (char *)NULL)
+      if (this_string == (char *)NULL)
 	done = 1;
       else
 	{
-	  this = mhtml_evaluate_string (this);
-	  that = mhtml_evaluate_string (that);
+	  this_string = mhtml_evaluate_string (this_string);
+	  with_that = mhtml_evaluate_string (with_that);
 
-	  if (this)
+	  if (this_string)
 	    page_subst_in_page_pivot
-	      (top_page->page, this, that, &(top_page->start));
+	      (top_page->page, this_string, with_that, &(top_page->start));
 
-	  if (this) free (this);
-	  if (that) free (that);
+	  xfree (this_string);
+	  xfree (with_that);
 	}
     }
 
@@ -607,7 +661,7 @@ pf_page_insert (PFunArgs)
 }
 #endif /* NOT_AT_THIS_TIME */
 
-DEFUN (pf_redirect, url,
+DEFUN (pf_redirect, url &key target,
 "Immediately return the HTTP <b>Location</b> directive with <var url>.
 This command is extremely useful in conjunction with <funref variables
 var-case>; in effect, that usage creates a dispatch table.
@@ -624,12 +678,18 @@ the following redirections are seen:
   <redirect ../foo.mhtml>     --> http://www.metahtml.com/foo.mhtml
   <redirect /bar/foo.mhtml>   --> http://www.metahtml.com/bar/foo.mhtml
   <redirect http://www.bsdi.com> --> http://www.bsdi.com
-</example>")
+</example>
+
+If the keyword argument <var target=XXX> is supplied, it is the name of
+the browser window in which to display the new page.  If one is working
+with frames, this can be very useful: a target of \"_top\" removes all
+frames, a target of \"_new\" creates a new window, etc.")
 {
   PAGE *new_page;
   char *arg = (char *)NULL;
   char *protocol = pagefunc_get_variable ("env::server_protocol");
   char *protover = pagefunc_get_variable ("env::protocol_version");
+  char *target = mhtml_evaluate_string (get_value (vars, "target"));
 
   if (!protocol) protocol = "HTTP";
   if (!protover) protover = "1.0";
@@ -699,20 +759,26 @@ the following redirections are seen:
 	}
 
 #if defined (macintosh)
-      bprintf (new_page,  "%s/%s 302 Found\nLocation: %s\n\n",
+      bprintf (new_page,  "%s/%s 302 Found\nLocation: %s\n",
 	       protocol, protover, arg + i);
 #else
       if (pagefunc_get_variable ("mhtml::unparsed-headers"))
-	bprintf (new_page, "%s/%s 302 Found\nLocation: %s\n\n",
+	bprintf (new_page, "%s/%s 302 Found\nLocation: %s\n",
 		 protocol, protover, arg + i);
       else
-	bprintf (new_page, "Location: %s\n\n", arg + i);
+	bprintf (new_page, "Location: %s\n", arg + i);
+
+      if (!empty_string_p (target))
+	bprintf (new_page, "Window-target: %s\n", target);
+
+      bprintf (new_page, "\n");
 #endif /* !macintosh */
     }
   else
     bprintf (new_page, "%s/%s.0 204 No Response\n\n", protocol, protover);
 
-  if (arg) free (arg);
+  xfree (arg);
+  xfree (target);
   page_return_this_page (new_page);
 }
 
@@ -898,3 +964,7 @@ interpreter, on those systems which have process IDs.")
 
   bprintf_insert (page, start, "%ld", (unsigned long)pid);
 }
+
+#if defined (__cplusplus)
+}
+#endif
