@@ -71,7 +71,11 @@
 #  endif
 #endif
 #include <sys/stat.h>
-#include <regex/regex.h>
+#if !defined (HAVE_REGEX_H)
+#  include <regex/regex.h>
+#else
+#  include <regex.h>
+#endif /* HAVE_REGEX_H */
 
 #include <xmalloc/xmalloc.h>
 #include <bprintf/bprintf.h>
@@ -80,6 +84,11 @@
 #include "mac_port.h"
 #endif
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 extern char LEFT_BRACKET, RIGHT_BRACKET;
 
 #undef whitespace
@@ -117,6 +126,13 @@ extern int errno;
 char *page_last_read_filename = (char *)NULL;
 static int plrf_size = 0;
 
+/* Local details about the page most recently loaded with page_read_template.*/
+struct stat page_last_finfo;
+
+/* The time of the most recent modification of all pages loaded through
+   page_read_template (). */
+unsigned long page_most_recent_modification_time = (unsigned long)0;
+
 /* Make PAGE contain CONTENTS, and nothing else. */
 void
 page_set_contents (PAGE *page, char *contents)
@@ -150,7 +166,6 @@ page_free_page (PAGE *page)
 PAGE *
 page_read_template (char *passed_filename)
 {
-  struct stat finfo;
   PAGE *buffer = (PAGE *)NULL;
   int result;
   char *filename = passed_filename;
@@ -204,13 +219,18 @@ page_read_template (char *passed_filename)
   }
 #endif /* macintosh */
 
-  result = stat (filename, &finfo);
+  result = stat (filename, &page_last_finfo);
 
-  if ((result != -1) && (!S_ISDIR (finfo.st_mode)))
+  if ((result != -1) && (!S_ISDIR (page_last_finfo.st_mode)))
     {
       int fd;
       int filename_len = strlen (filename);
 
+      if (page_most_recent_modification_time
+	  < (unsigned long)page_last_finfo.st_mtime)
+	page_most_recent_modification_time =
+	  (unsigned long)page_last_finfo.st_mtime;
+
       if (filename_len + 2 > plrf_size)
 	page_last_read_filename = 
 	  (char *)xrealloc (page_last_read_filename,
@@ -220,7 +240,7 @@ page_read_template (char *passed_filename)
       fd = os_open (filename, O_RDONLY, 0666);
       if (fd > -1)
 	{
-	  int filesize = (int) finfo.st_size;
+	  int filesize = (int) page_last_finfo.st_size;
 	  char *contents = (char *)xmalloc (1 + filesize);
 
 	  if (read (fd, contents, filesize) != filesize)
@@ -253,7 +273,7 @@ page_read_template (char *passed_filename)
    of characters added or deleted before it.  An NULL pointer means skip this
    calculation. */
 int
-page_subst_in_page_pivot (PAGE *page, char *this, char *with_that, int *pivot)
+page_subst_in_page_pivot (PAGE *page, char *ths, char *with_that, int *pivot)
 {
   regex_t re;
   regmatch_t offsets[10];
@@ -266,7 +286,7 @@ page_subst_in_page_pivot (PAGE *page, char *this, char *with_that, int *pivot)
   int ro_index = 0;
   int eflags = 0;
 
-  regcomp (&re, this, REG_EXTENDED);
+  regcomp (&re, ths, REG_EXTENDED);
 
   search_point = done = 0;
 
@@ -391,9 +411,9 @@ page_subst_in_page_pivot (PAGE *page, char *this, char *with_that, int *pivot)
    The substitutions happen in place, and the return value is the number of
    characters added or deleted.  THIS is a regular expression. */
 int
-page_subst_in_page (PAGE *page, char *this, char *with_that)
+page_subst_in_page (PAGE *page, char *ths, char *with_that)
 {
-  return (page_subst_in_page_pivot (page, this, with_that, (int *)NULL));
+  return (page_subst_in_page_pivot (page, ths, with_that, (int *)NULL));
 }
 
 /* Return the offset of STRING in PAGE.  Start the search at START.
@@ -1028,6 +1048,20 @@ page_find_tag_end (PAGE *page, int start)
 	      quote_depth = !quote_depth;
 	      break;
 
+	    case ';':
+	      if (((i + 2) < page->bindex) &&
+		  (page->buffer[i + 1] == ';') &&
+		  (page->buffer[i + 2] == ';'))
+		{
+		  /* Found a comment.  Skip it. */
+		  i += 2;
+		  while ((i < page->bindex) && (page->buffer[i] != '\n')) i++;
+
+		  /* Our index is on the newline character that ends this line.
+		     Just continue on. */
+		}
+	      break;
+
 	    default:
 	      break;
 	    }
@@ -1370,3 +1404,6 @@ page_pdl_page (int offset)
   return (&page_pdl[offset]);
 }
 
+#if defined (__cplusplus)
+}
+#endif
