@@ -31,14 +31,19 @@
 #define COMPILING_PARSER_C 1
 #include "language.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 char LEFT_BRACKET = '<';
 char RIGHT_BRACKET = '>';
 
 extern void initialize_external_functions (Package *p);
 
 char *metahtml_copyright_string = "\
-Copyright (C) 1995, 1996, Brian J. Fox\n\
-Copyright (C) 1996, 1997 Universal Access Inc.";
+Copyright (C) 1995, 1998, Brian J. Fox\n\
+Copyright (C) 1996, 1998, Universal Access Inc.";
 
 /* The parser's idea of what the current line number is. */
 int parser_current_lineno = 0;
@@ -46,13 +51,18 @@ int parser_current_lineno = 0;
 /* Globally known variable holds onto to the reserved words. */
 Package *mhtml_function_package = (Package *)NULL;
 
-/* Macro writing and processing. */
+/* Globally known variable is the package containing user-defined functions. */
 Package *mhtml_user_keywords = (Package *)NULL;
 
 /* Variable specifically for the mdb debugger.  This allows the setting
    of a callback function when an mdb breakpoint is hit. */
 DEBUGGER_CALLBACK_FUNCTION *mhtml_debug_callback_function = NULL;
 
+/* Variable specifically for interrupting the parser.  When non-zero,
+   the parser stops dead in its tracks.  MDB uses this. */
+int mhtml_parser_interrupted = 0;
+MHTML_PARSER_CALLBACK_FUNCTION *mhtml_parser_callback_function = NULL;
+
 /* The "DEFAULT" package. */
 Package *PageVars = (Package *)NULL;
 
@@ -125,6 +135,15 @@ int mhtml_gather_documentation = 0;
 int mhtml_profiling_functions = 0;
 #endif
 
+/* Gets 1 when mhtml::warn-on-redefine-primitive has a value, 0 otherwise. */
+int mhtml_warn_on_redefine_primitive = 0;
+
+/* Gets 1 when mhtml::warn-on-redefine-function has a value, 0 otherwise. */
+int mhtml_warn_on_redefine = 0;
+
+/* Gets 1 when mhtml::verbose-error-reporting has a value, 0 otherwise. */
+int mhtml_verbose_error_reporting = 0;
+
 void
 pagefunc_initialize_notifiers (void)
 {
@@ -138,6 +157,12 @@ pagefunc_initialize_notifiers (void)
   symbol_notify_value (sym, &mhtml_parser_trace_mode);
   sym = symbol_intern ("mhtml::gather-documentation");
   symbol_notify_value (sym, &mhtml_gather_documentation);
+  sym = symbol_intern ("mhtml::warn-on-redefine-primitive");
+  symbol_notify_value (sym, &mhtml_warn_on_redefine_primitive);
+  sym = symbol_intern ("mhtml::warn-on-redefine");
+  symbol_notify_value (sym, &mhtml_warn_on_redefine);
+  sym = symbol_intern ("mhtml::verbose-error-reporting");
+  symbol_notify_value (sym, &mhtml_verbose_error_reporting);
 #if defined (METAHTML_PROFILER)
   sym = symbol_intern ("mhtml::profile-functions");
   symbol_notify_value (sym, &mhtml_profiling_functions);
@@ -812,6 +837,90 @@ parser_trace_indentation (void)
   return (blanks + (blanks_size - indent_amount));
 }
 
+/* #define METAHTML_READER_MACROS 1 */
+/* This can't work unless we rewrite read_sexp_1 from scratch.  In
+   addition, it isn't the right thing for a passive-parsing language.
+   I've left the code here to discourage other people from asking for
+   it as a `feature'. */
+#undef METAHTML_READER_MACROS
+#if defined (METAHTML_READER_MACROS)
+static int
+reader_macro (PAGE *page, int point)
+{
+  int result = 0;
+
+  if (page->buffer[point] == '$')
+    {
+      /* Surround the following symbol with <get-var ...>. */
+      register int i;
+      register int found = 0;
+
+      for (i = point + 1; ((i < page->bindex) && ((point - i) < 50)); i++)
+	{
+	  register unsigned char c = (unsigned char)page->buffer[i];
+
+	  if ((c > 'z') || (c < 'A') || ((c > 'Z') && (c < 'a')))
+	    {
+	      found++;
+	      break;
+	    }
+	}
+
+      if (found && ((i - point) != 1))
+	{
+	  bprintf_delete_range (page, point, point + 1);
+	  i--;
+	  bprintf_insert (page, i, ">");
+	  bprintf_insert (page, point, "<get-var ");
+	  result = 1;
+	}
+    }
+
+  return (result);
+}
+#endif
+
+/* A little help for the debugger. */
+int debugging_with_mdb = 0;
+
+char **pushed_function_names = (char **)NULL;
+int pushed_function_names_index = 0;
+int pushed_function_names_slots = 0;
+static void
+push_function_name (char *name)
+{
+  if (pushed_function_names_index + 2 > pushed_function_names_slots)
+    pushed_function_names = (char **)xrealloc
+      (pushed_function_names, (pushed_function_names_slots += 10)
+       * sizeof (char *));
+
+  pushed_function_names[pushed_function_names_index++] = name;
+  pushed_function_names[pushed_function_names_index] = (char *)NULL;
+}
+
+static void
+pop_function_name (void)
+{
+  pushed_function_names_index--;
+  pushed_function_names[pushed_function_names_index] = (char *)NULL;
+}
+
+char **
+mdb_pushed_function_names (void)
+{
+  return (pushed_function_names);
+}
+
+void
+mdb_reset_pushed_functions (void)
+{
+  if (pushed_function_names)
+    {
+      pushed_function_names_index = 0;
+      pushed_function_names[0] = (char *)NULL;
+    }
+}
+
 /* Actually process PAGE in place.  The result of processing PAGE is placed
    within PAGE.  This is likely to change shortly, when we pre-parse the
    PAGE and write sequential output to a different destination. */
@@ -842,8 +951,29 @@ page_process_page_internal (PAGE *page)
 	  search_start_modified = -1;
 	}
 
+      if (mhtml_parser_interrupted)
+	{
+	  if (mhtml_parser_callback_function)
+	    {
+	      mhtml_parser_interrupted = 0;
+	      (*mhtml_parser_callback_function) ();
+	    }
+	  else
+	    return;
+	}
+
       for (i = search_start; i < page->bindex; i++)
 	{
+#undef METAHTML_READER_MACROS /* See comment at reader_macro () */
+#if defined METAHTML_READER_MACROS
+	  if (reader_macro (page, i))
+	    {
+	      /* reader_macro () has already expanded the read macro.
+		 Just read again. */
+	      i--;
+	      continue;
+	    }
+#endif
 	  if ((page->buffer[i] == LEFT_BRACKET) || (page->buffer[i] == '<'))
 	    break;
 
@@ -866,14 +996,21 @@ page_process_page_internal (PAGE *page)
 					    page->buffer[i + 1])))
 		    i++;
 
+		  /* Handle CR/LF. */
+		  if (page->buffer[i] == '\r') i++;
+
 		  parser_current_lineno++;
 
 		  /* If the very next item is another semi-colon comment,
 		     delete it in one chunk with the current one. */
-		  if (((i + 2) < page->bindex) &&
+		  if (((i + 3) < page->bindex) &&
 		      (page->buffer[i + 1] == ';') &&
-		      (page->buffer[i + 2] == ';'))
-		    goto get_another_comment;
+		      (page->buffer[i + 2] == ';') &&
+		      (page->buffer[i + 3] == ';'))
+		    {
+		      i++;
+		      goto get_another_comment;
+		    }
 
 		  bprintf_delete_range (page, marker, i + 1);
 		  i = marker - 1;
@@ -984,6 +1121,23 @@ page_process_page_internal (PAGE *page)
 			      thisfile ? thisfile : "???",
 			      LEFT_BRACKET, desc->tag, RIGHT_BRACKET);
 
+		  if (mhtml_verbose_error_reporting)
+		    {
+		      int pindex;
+		      char partial[100];
+
+		      for (pindex = 0;
+			   (pindex < 99) && (start + pindex < page->bindex);
+			   pindex++)
+			partial[pindex] = page->buffer[start + pindex];
+
+		      partial[pindex] = '\0';
+
+		      page_debug
+			("[Text Following Missing Tag:\n%s\nEnd of Text]",
+			 partial);
+		    }
+
 		  if (syntax_checking)
 		    {
 		      syntax_failure = 1;
@@ -1120,10 +1274,11 @@ page_process_page_internal (PAGE *page)
 			    page_debug ("Entering %c%s %s%c",
 					LEFT_BRACKET, desc->tag,
 					display_body, RIGHT_BRACKET);
-
 			}
 
 		      parser_trace_indentation_counter++;
+		      if (debugging_with_mdb)
+			push_function_name (desc->tag);
 		      {
 			int debug_continue = 1;
 
@@ -1223,6 +1378,8 @@ page_process_page_internal (PAGE *page)
 					RIGHT_BRACKET);
 			  free (display_body);
 			}
+		      if (debugging_with_mdb)
+			pop_function_name ();
 
 		      page_free_page (body);
 		    }
@@ -1305,6 +1462,54 @@ mhtml_find_user_function (char *name)
   return ((UserFunction *)NULL);
 }
 
+#define REDEFINED_PRIMITIVE 0x01
+#define REDEFINED_UFUNCTION 0x02
+
+/* Given that we are about to define a new function, check to see if
+   the definition would overwrite an existing definition.  We only
+   perform this check if the user has set one of the variables
+   mhtml::warn-on-redefine-primitive or mhtml::warn-on-redefine. */
+void
+mhtml_maybe_warn_redefine (char *name, int type)
+{
+  if (mhtml_warn_on_redefine_primitive || mhtml_warn_on_redefine)
+    {
+      int redefined = 0;
+      PFunDesc *prim = pagefunc_get_descriptor (name);
+
+      if (prim != (PFunDesc *)NULL)
+	redefined = REDEFINED_PRIMITIVE;
+
+      if (mhtml_warn_on_redefine)
+	{
+	  UserFunction *ufun = mhtml_find_user_function (name);
+
+	  if (ufun != (UserFunction *)NULL)
+	    redefined |= REDEFINED_UFUNCTION;
+	}
+
+      if (redefined)
+	{
+	  char *definer = "primitive?";
+
+	  switch (type)
+	    {
+	    case user_DEFUN: definer = "defun"; break;
+	    case user_SUBST: definer = "defsubst"; break;
+	    case user_MACRO: definer = "defmacro"; break;
+	    }
+
+	  if (redefined & REDEFINED_PRIMITIVE)
+	    page_syserr ("WARNING: <%s %s...> redefines a primitive function",
+			 definer, name);
+
+	  if (redefined & REDEFINED_UFUNCTION)
+	    page_syserr ("WARNING: <%s %s...> redefines a user function",
+			 definer, name);
+	}
+    }
+}
+
 /* Add or replace a function of TYPE with NAME, BODY in the
    *user-functions* package. The definition is modified by variable
    names and values specified in the package passed in VARS. */
@@ -1319,6 +1524,11 @@ mhtml_add_user_function (int type, char *name, char *body, Package *vars)
   int np_size = 0;
   int np_index = 0;
 
+  /* If this is a redefinition of a primitive, and we have those warnings
+     turned on, then produce an error message in system-error-output. */
+  if (mhtml_warn_on_redefine_primitive || mhtml_warn_on_redefine)
+    mhtml_maybe_warn_redefine (name, type);
+
   if (type == user_DEFUN)
     {
       body_whitespace = "delete";
@@ -1769,6 +1979,8 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 	    {
 	      int gathering_optional = 0;
 	      int gathering_keys = 0;
+	      int gathering_body = 0;
+	      int unevalled = 0;
 	      int arg_offset = 0;
 
 	      if ((packname != (char *)NULL) &&
@@ -1796,6 +2008,20 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 		      continue;
 		    }
 
+		  /* Handle &unevalled argument. */
+		  if (strcasecmp (param, "&unevalled") == 0)
+		    {
+		      unevalled++;
+		      continue;
+		    }
+
+		  /* Handle &body argument. */
+		  if (strcasecmp (param, "&body") == 0)
+		    {
+		      gathering_body++;
+		      continue;
+		    }
+
 		  /* Handle &rest argument. */
 		  if (strcasecmp (param, "&rest") == 0)
 		    {
@@ -1821,18 +2047,16 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 				{
 				  char *setval = value;
 
-				  /* if (uf->type == user_DEFUN) */
+				  if (!unevalled)
 				    {
 				      setval = mhtml_evaluate_string (setval);
 				      symbol_add_value (sym, setval);
 				      free (setval);
 				    }
-				    /*
 				  else
 				    {
 				      symbol_add_value (sym, setval);
 				    }
-				    */
 				  array_offset++;
 				}
 			    }
@@ -1849,13 +2073,14 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 				{
 				  char *setval = gather->buffer;
 
-				  /* if (uf->type == user_DEFUN) */
+				  if (!unevalled)
 				    {
 				      setval = mhtml_evaluate_string (setval);
 				      symbol_add_value (sym, setval);
 				      free (setval);
 				    }
-				    /* else symbol_add_value (sym, setval); */
+				  else
+				    symbol_add_value (sym, setval);
 				}
 
 			      bprintf_free_buffer (gather);
@@ -1869,23 +2094,46 @@ mhtml_execute_function (UserFunction *uf, PFunArgs, char *attr)
 		    }
 
 		  /* Get argument value of the right type. */
-		  if (gathering_keys)
-		    value = forms_get_tag_value_in_package (vars, param);
+		  if (gathering_body)
+		    {
+		      gathering_body = 0;
+
+		      if (!empty_string_p (body->buffer))
+			{
+			  j = 0;
+			  if (uf->type != user_MACRO)
+			    while (whitespace (body->buffer[j])) j++;
+
+			  value = body->buffer + j;
+			}
+		      else
+			value = (char *)NULL;
+		    }
+		  else if (gathering_keys)
+		    {
+		      if (vars != (Package *)NULL)
+			value = forms_get_tag_value_in_package (vars, param);
+		      else
+			value = (char *)NULL;
+		    }
 		  else
 		    value = get_positional_arg (vars, arg_offset++);
 
 		  if (value != (char *)NULL)
 		    {
-		      /* if (uf->type == user_DEFUN) */
+		      if (!unevalled)
 			{
 			  value = mhtml_evaluate_string (value);
 			  forms_set_tag_value_in_package
 			    (param_package, param, value);
 			  free (value);
 			}
-			/* else forms_set_tag_value_in_package
-			   (param_package, param, value); */
+		      else forms_set_tag_value_in_package
+			     (param_package, param, value);
 		    }
+		  /* Reset unevalled -- it must be specified for each
+		     variable which is to be unevalled. */
+		  unevalled = 0;
 		}
 	    }
 
@@ -2029,6 +2277,8 @@ mhtml_canonicalize_file_name (char *input, char *docroot, char *relpref)
 		for (last_slash = i; last_slash > 0; last_slash--)
 		  if (workbuff[last_slash] == '/')
 		    break;
+
+		if (last_slash < 0) last_slash = 0;
 	      }
 	    else
 	      last_slash = i;
@@ -2401,3 +2651,7 @@ number_p (char *string)
 
   return (result);
 }
+
+#if defined (__cplusplus)
+}
+#endif
