@@ -31,6 +31,11 @@
 #include "language.h"
 #include "symdump.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 static void pf_eval (PFunArgs);
 static void pf_read (PFunArgs);
 static void pf_write_package_file (PFunArgs);
@@ -94,6 +99,10 @@ pf_eval (PFunArgs)
     }
 }
 
+DEFUNX (pf_%%read, string,
+"Reads one symbolic expression from <var string> and returns it.
+There isn't the slightest possibility that you need this in your
+programs.")
 static void
 pf_read (PFunArgs)
 {
@@ -115,6 +124,11 @@ pf_read (PFunArgs)
     }
 }
 
+DEFUNX (pf_%%write_package_file, filename &rest packages,
+"Writes the contents of <var packages> to the file specified by <var filename>.
+This function is used internally when creating libraries.  There isn't the
+slightest possibility that you need this in your programs -- if you think
+you do, you probably simply want to use <b>mklib</b>.")
 static void
 pf_write_package_file (PFunArgs)
 {
@@ -158,6 +172,12 @@ pf_write_package_file (PFunArgs)
     bprintf_insert (page, start, "%d", packages_written);
 }
 
+DEFUNX (pf_%%read_package_file, filename,
+"Reads package contents from the file specified by <var filename>, which
+had best be created using <funref primitive-operators %%write-package-file>.
+This function is used internally when loading libraries.  There isn't the
+slightest possibility that you need this in your programs -- if you think
+you do, you probably simply want to use <funref file-operators require>.")
 static void
 pf_read_package_file (PFunArgs)
 {
@@ -210,6 +230,12 @@ pf_read_package_file (PFunArgs)
     }
 }
 
+DEFUNX (pf_%%switch_user, newuser,
+"Changes the current user to <var newuser>, and returns \"true\" if the
+switch was successful.  This can only take place if the running Meta-HTML is
+running under the user ID of the superuser (root).  After calling this
+function, it is impossible to switch back to another user, so it really
+isn't of much use in Meta-HTML programs.")
 static void
 pf_switch_user (PFunArgs)
 {
@@ -240,9 +266,9 @@ pf_switch_user (PFunArgs)
     }
 }
 
-/* <%%the-page [varname]>
-   Places the current page into VARNAME as a binary variable, or,
-   returns the current page as text if VARNAME is not supplied. */
+DEFUNX (pf_%%the_page,  &optional varname,
+"Places the current page into <var varname> as a binary variable, or,
+returns the current page as text if <var varname> is not supplied.")
 static void
 pf_the_page (PFunArgs)
 {
@@ -271,15 +297,19 @@ pf_the_page (PFunArgs)
   xfree (varname);
 }
 
-/* <%%point> --> Returns the current parser marker in the page.
-   CAVEAT!  Currently, can only be called at top level. */
+DEFUNX (pf_%%point, ,
+"Returns the current parser marker in the page.
+<b>CAVEAT</b>!  Currently, can only be called at top level.")
 static void
 pf_point (PFunArgs)
 {
   bprintf_insert (page, start, "%d", start);
 }
 
-/* <%%function-documentation user-function> --> documentation strings. */
+DEFUNX (pf_%%function_documentation,  user-function,
+"Returns the documentation for <var user-function>.  Only works if the
+variable <var mhtml::gather-documentation> was set at the time the
+<var user-function> was defined.")
 static void
 pf_function_documentation (PFunArgs)
 {
@@ -306,7 +336,10 @@ pf_function_documentation (PFunArgs)
   xfree (name);
 }
 
-/* <%%function-arguments user-function> --> Argument strings. */
+DEFUNX (pf_%%function_arguments, user-function,
+"Returns an array of the formal parameters for <var user-function>.
+Essentially, this returns exactly what was entered at the time the
+function definition was defined.")
 static void
 pf_function_arguments (PFunArgs)
 {
@@ -334,7 +367,10 @@ pf_function_arguments (PFunArgs)
   xfree (name);
 }
 
-/* <%%quote-for-set-var arg...> --> "<evaluated arg>..."> */
+DEFUNX (pf_%%quote_for_set_var, &rest body,
+"After evaluating <var body>, the results are quoted in such a way that
+Meta-HTML will treat it as one argument.  Used internally by the function
+invoker.")
 static void
 pf_quote_for_setvar (PFunArgs)
 {
@@ -353,3 +389,7 @@ pf_quote_for_setvar (PFunArgs)
 	}
     }
 }
+
+#if defined (__cplusplus)
+}
+#endif
