@@ -32,6 +32,11 @@
 #include "language.h"
 #include "symdump.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 /* The name of the variable which holds the already found filenames. */
 #define REQUIRE_LOADED "MHTML::REQUIRE-LOADED"
 
@@ -146,6 +151,16 @@ require_search (char *root, char *leaf, int *filetype)
       free (temp);
     }
 
+  if ((result != (char *)NULL) &&
+      ((unsigned long)finfo.st_mtime > page_most_recent_modification_time))
+    {
+      char digits[40];
+
+      page_most_recent_modification_time = (unsigned long)finfo.st_mtime;
+      sprintf (digits, "%ld", (unsigned long)finfo.st_mtime);
+      pagefunc_set_variable ("mhtml::last-mmodification-time", digits);
+    }
+
   return (result);
 }
 
@@ -239,6 +254,17 @@ array variable <varref mhtml::require-loaded>.")
 	case require_TYPE_INCLUDE:
 	  {
 	    PAGE *file_contents = page_read_template (pathname);
+	    char *incpref = pagefunc_get_variable ("%%::incpref");
+	    char *relpref = pagefunc_get_variable ("%%::relpref");
+	    char *temp;
+
+	    if (!incpref)
+	      incpref = pagefunc_get_variable ("mhtml::include-prefix");
+	    if (!relpref)
+	      relpref = pagefunc_get_variable ("mhtml::relative-prefix");
+
+	    if (debug_level > 5)
+	      page_debug ("<require %s> --> Found %s", arg, pathname);
 
 	    if (file_contents != (PAGE *)NULL)
 	      {
@@ -252,21 +278,33 @@ array variable <varref mhtml::require-loaded>.")
 		bprintf (file_contents, "<*parser*::pop-file>");
 		bprintf (file_contents, "<if <function-def .libinit> ");
 		bprintf (file_contents, "\"<.libinit><undef .libinit>\">");
+		bprintf_insert_binary (page, start, file_contents->buffer,
+				       file_contents->bindex);
 
-		/* Manually insert the file instead of letting bprintf
-		   do it for us.  This is because the file could contain
-		   binary data, and then file->bindex wouldn't necessarily
-		   reflect the length of what was inserted. */
-		if ((file_contents->bindex + page->bindex) >= page->bsize)
-		  page->buffer = (char *)xrealloc
-		  (page->buffer, (page->bsize += (file_contents->bindex + 100)));
+		temp = strstr (pathname, incpref);
 
-		memmove (page->buffer + start + file_contents->bindex,
-			 page->buffer + start, (page->bindex + 1) - start);
+		if (temp != (char *)NULL)
+		  {
+		    char *last;
 
-		memcpy (page->buffer + start, file_contents->buffer,
-			file_contents->bindex);
-		page->bindex += file_contents->bindex;
+		    temp += strlen (incpref);
+		    last = strrchr (temp, '/');
+
+		    if (last != (char *)NULL)
+		      {
+			*last = '\0';
+
+			/* Insert instructions to restore the state. */
+			bprintf_insert
+			  (page, start + file_contents->bindex,
+			   "<set-var %%%%::relpref=%s>",
+			   relpref ? relpref : "");
+			pagefunc_set_variable ("%%::relpref", temp);
+			if (debug_level > 5)
+			  page_debug ("REQUIRE: Changed %%::relpref to %s",
+				      temp);
+		      }
+		  }
 #endif /* BINARY_COMPATIBLE */
 		page_free_page (file_contents);
 	      }
@@ -323,6 +361,10 @@ array variable <varref mhtml::require-loaded>.")
 	  break;
 	}
     }
-  if (pathname) free (pathname);
-  if (arg) free (arg);
+  xfree (pathname);
+  xfree (arg);
 }
+
+#if defined (__cplusplus)
+}
+#endif
