@@ -48,6 +48,10 @@
 #  include "sessiondb.h"
 #endif /* !HAVE_SHM_SESSIONS */
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 
 /* ************************************************************ */
 /*								*/
@@ -65,6 +69,11 @@ static SESSION_INFO *session_sequential_access (DBFILE, SESSION_INFO *);
 static void session_delete (session_id_t sid);
 static SESSION_INFO *session_get_info_internal (session_id_t sid);
 static void session_put_info_internal (SESSION_INFO *info);
+#if defined (MHTML_CRYPTOGRAPHY)
+static unsigned char *session_encrypt (char *key, unsigned char *data, int *l);
+static unsigned char *session_decrypt (char *key, unsigned char *data, int *l);
+char *session_encryption_key = (char *)NULL;
+#endif /* MHTML_CRYPTOGRAPHY */
 
 /* ************************************************************ */
 /*								*/
@@ -447,9 +456,9 @@ session_free (SESSION_INFO *info)
 {
   if (info != (SESSION_INFO *)NULL)
     {
-      if (info->sid) free (info->sid);
-      if (info->key) free (info->key);
-      if (info->data) free (info->data);
+      xfree (info->sid);
+      xfree (info->key);
+      xfree (info->data);
       free (info);
     }
 }
@@ -681,19 +690,66 @@ get_lisp_string (char *tag, WispObject *list)
   return (value);
 }
 
+#if defined (MHTML_CRYPTOGRAPHY)
+extern unsigned char *triple_des (unsigned char *, char *, int *, int);
+
+static unsigned char *
+session_encrypt (char *key, unsigned char *data, int *len)
+{
+  unsigned char *result = (unsigned char *)NULL;
+
+  if (*len != 0)
+    result = triple_des (data, key, len, 1);
+
+  return (result);
+}
+
+static unsigned char *
+session_decrypt (char *key, unsigned char *data, int *len)
+{
+  unsigned char *result = (unsigned char *)NULL;
+
+  if (*len != 0)
+    result = triple_des (data, key, len, 0);
+
+  return (result);
+}
+#endif /* MHTML_CRYPTOGRAPHY */
+
 static char *
 encode_data (unsigned char *data, int length)
 {
   register int i;
-  char *result;
+  unsigned char *result;
+
+#if defined (MHTML_CRYPTOGRAPHY)
+  int free_data = 0;
+
+  /* If encrypted sessioning is turned on, then handle the
+     encryption of the data now. */
+  if ((length != 0) &&
+      (session_encryption_key != (char *)NULL) &&
+      (session_encryption_key[0] != '\0'))
+    {
+      int len = 1 + length;
+      data = session_encrypt (session_encryption_key, data, &len);
+      length = len;
+      free_data++;
+    }
+#endif /* MHTML_CRYPTOGRAPHY */
+
+  result = (unsigned char *)xmalloc (1 + (2 * length));
 
-  result = (char *)xmalloc (1 + (2 * length));
   for (i = 0; i < length; i++)
-    sprintf (result + (2 * i), "%02x", data[i]);
+    sprintf ((char *)result + (2 * i), "%02x", data[i]);
 
   result[2 * i] = '\0';
 
-  return (result);
+#if defined (MHTML_CRYPTOGRAPHY)
+  if (free_data) free (data);
+#endif
+
+  return ((char *)result);
 }
 
 static char *
@@ -724,6 +780,23 @@ decode_data (char *data)
     }
 
   result[i / 2] = '\0';
+
+#if defined (MHTML_CRYPTOGRAPHY)
+  /* If encrypted sessioning is turned on, then handle the
+     decryption of the data now. */
+  if ((i != 0) &&
+      (session_encryption_key != (char *)NULL) &&
+      (session_encryption_key[0] != '\0'))
+    {
+      int len = i / 2;
+      unsigned char *decrypted;
+
+      decrypted = session_decrypt (session_encryption_key, result, &len);
+      free (result);
+      result = decrypted;
+    }
+#endif /* MHTML_CRYPTOGRAPHY */
+
   return ((char *)result);
 }
 
@@ -789,9 +862,13 @@ info_to_content (SESSION_INFO *info)
 
   content = (DBOBJ *)xmalloc (sizeof (DBOBJ));
   content->data = (unsigned char *)buffer->buffer;
-  content->length = 1 + strlen (content->data);
+  content->length = 1 + strlen ((char *)content->data);
 
   free (buffer);
 
   return (content);
 }
+
+#if defined (__cplusplus)
+}
+#endif
