@@ -31,6 +31,11 @@
 #include "language.h"
 #include "session_data.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 /************************************************************/
 /*							    */
 /*		Session Manipulation Functions		    */
@@ -51,6 +56,9 @@ static void pf_sessions_of_key (PFunArgs);
 static void pf_get_session_data (PFunArgs);
 static void pf_set_session_data (PFunArgs);
 static void pf_set_session_db (PFunArgs);
+#if defined (MHTML_CRYPTOGRAPHY)
+static void pf_set_session_encryption_key (PFunArgs);
+#endif /* MHTML_CRYPTOGRAPHY */
 
 static PFunDesc func_table[] =
 {
@@ -70,6 +78,10 @@ static PFunDesc func_table[] =
   { "SET-SESSION-DATA", 0, 0, pf_set_session_data },
   { "SET-SESSION-DB",	0, 0, pf_set_session_db },
 
+#if defined (MHTML_CRYPTOGRAPHY)
+  { "SET-SESSION-ENCRYPTION-KEY", 0, 0, pf_set_session_encryption_key },
+#endif /* MHTML_CRYPTOGRAPHY */
+
   /* Deprecate this as soon as possible. */
   { "SESSION-DATA-ALIST", 0, 0, pf_get_session_data },
 
@@ -459,11 +471,9 @@ default::sid> (i.e., the current session ID).")
   if (passed != (char *)NULL) free (passed);
 }
 
-static int
-valid_alist_p (char *alist)
-{
-  return (1);
-}
+/* Punt.  Not because we can't do it, but because it doesn't make good
+   sense to do it in set_session_data(). */
+#define valid_alist_p(string) 1
   
 /* <set-session-data [alist] [sid]> --> Set session info from ALIST. */
 DEFUN (pf_set_session_data, alist &optional sid,
@@ -497,12 +507,12 @@ SESSION-OPERATORS session-export> should be more than sufficient.")
 
 	  if (valid_alist_p (alist))
 	    {
-	      info->data = strdup (alist);
+	      info->data = (unsigned char *)strdup (alist);
 	      info->length = strlen (alist);
 	    }
 	  else
 	    {
-	      info->data = (char *)NULL;
+	      info->data = (unsigned char *)NULL;
 	      info->length = 0;
 	    }
 
@@ -876,10 +886,11 @@ Server and CGI Engine, see the section <secref server-variables>.")
     bprintf_insert (page, start, "%s", code);
 }
 
-/* <set-session-db pathname> --> Dynamically change the location of the
-   file used to store session information. */
-static void
-pf_set_session_db (PFunArgs)
+DEFUN (pf_set_session_db, pathname,
+"Dynamically change the location of the file used to store session information.
+You probably don't need this in your programs -- it is used within the server
+and engine configuration files to specify the location of your session
+database.")
 {
   char *session_db_file = mhtml_evaluate_string (get_positional_arg (vars, 0));
 
@@ -894,3 +905,41 @@ pf_set_session_db (PFunArgs)
 
   xfree (session_db_file);
 }
+
+#if defined (MHTML_CRYPTOGRAPHY)
+extern char *session_encryption_key;
+
+DEFUN (pf_set_session_encryption_key, keystring,
+"With non-empty argument <var keystring>, turns on the triple-DES
+encryption of data saved to the session database.  The data is
+encrypted and decrypted using <var key>.
+
+This could usefully be placed in the \"mhttpd.conf\" or \"engine.conf\"
+file with a statement similar to the following:
+
+<example>
+  <set-session-encryption-key <make-identifier 128>>
+</example>
+
+Don't turn this on unless you really have to, as it (obviously) affects
+the performance of the session database.  (Each access to the database
+results in a full call to triple ECB DES encryption, using the entire
+contents of the session, and encrypting/decryption with a random 128-bit
+key.)")
+{
+  char *key = mhtml_evaluate_string (get_positional_arg (vars, 0));
+
+  xfree (session_encryption_key);
+  session_encryption_key = (char *)NULL;
+
+  if (!empty_string_p (key))
+    session_encryption_key = strdup (key);
+
+  xfree (key);
+}
+#endif /* MHTML_CRYPTOGRAPHY */
+
+
+#if defined (__cplusplus)
+}
+#endif
