@@ -6,10 +6,10 @@ DEFUN (pf_array-delete-index,  index arrayvar,
  so that the array ends up with one less element than it had before.
 ")
 DOC_SECTION (MISCELLANEOUS-TAGS)
-DEFUN (pf_comma-separated,  array,
+DEFUN (pf_comma-separated,  arrayvar,
 " Produce a human readable string of the elements in the array
- variable <var array> separated by commas where appropriate, and
- the word \"and\" after the penultimate item.
+ variable <var arrayvar> separated by commas where appropriate,
+ and with the word \"and\" after the penultimate item.
 
 ")
 DOC_SECTION (ARITHMETIC-OPERATORS)
@@ -60,6 +60,31 @@ DEFUN (pf_date::canonicalize,  date-string,
  </complete-example>
  Start by removing the day-of-week.
 ")
+DOC_SECTION (HTML-HELPERS)
+DEFUN (pf_date::days-in-month,  month,
+" Return the number of days in <var month>.
+ <var month> is the name of a month, such as \"Apr\" or \"april\".
+")
+DOC_SECTION (HTML-HELPERS)
+DEFUN (pf_date::long-month-name,  month,
+" Return the canonical long name of <var month>.
+ <var month> is the name of a month, such as \"Apr\" or \"april\".
+")
+DOC_SECTION (HTML-HELPERS)
+DEFUN (pf_date::seconds-in-month,  month,
+" Return the number of seconds in <var month>.
+ <var month> is the name of a month, such as \"Apr\" or \"april\".
+")
+DOC_SECTION (HTML-HELPERS)
+DEFUN (pf_date::short-month-name,  month,
+" Return the canonical short name of <var month>.
+ <var month> is the name of a month, such as \"Apr\" or \"april\".
+")
+DOC_SECTION (LANGUAGE-OPERATORS)
+DEFUN (pf_defined?,  name,
+" Returns \"true\" if <var name> is defined as a Meta-HTML primitive or
+ a user-defined function, or the empty string otherwise.
+")
 DOC_SECTION (FILE-OPERATORS)
 DEFUN (pf_dir::basename,  pathname,
 " Return the \"basename\" of <var pathname>.  This simply returns the last
@@ -67,7 +92,7 @@ DEFUN (pf_dir::basename,  pathname,
 
  Example:
  <complete-example>
- <dir::basename /foo/bar/baz>
+ <dir::basename /foo/bar/baz.dtd>
  </complete-example>
 ")
 DOC_SECTION (FILE-OPERATORS)
@@ -87,6 +112,16 @@ DEFUN (pf_dir::canonicalize-pathname,  path,
  </complete-example>
 ")
 DOC_SECTION (FILE-OPERATORS)
+DEFUN (pf_dir::dirname,  pathname,
+" Return the directory part of <var pathname>.  This simply
+ returns everything but the last component of <var pathname>.
+
+ Example:
+ <complete-example>
+ <dir::dirname /foo/bar/baz.dtd>
+ </complete-example>
+")
+DOC_SECTION (FILE-OPERATORS)
 DEFUN (pf_dir::filename-extension,  filename,
 " Return the filename extension of <var filename>, without the leading
  period.  Returns the empty string if <var filename> doesn't have an
@@ -99,6 +134,14 @@ DEFUN (pf_dir::filename-extension,  filename,
  </example>
 ")
 DOC_SECTION (FILE-OPERATORS)
+DEFUN (pf_dir::handle-dots,  path,
+" Return <var path>, resolving any \"./\" or \"../\" found within.
+ Example:
+ <complete-example>
+ <dir::handle-dots /foo/bar/../baz/./../below-foo>
+ </complete-example>
+")
+DOC_SECTION (FILE-OPERATORS)
 DEFUN (pf_dir::names-in-dir,  dir type matching,
 " Return an array of the fully qualified pathnames in <var dir> which
  are of type <var type> (either \"<code>FILE</code>\" or
@@ -110,6 +153,25 @@ DEFUN (pf_dir::names-in-dir,  dir type matching,
  <var matching> defaults to all possible filenames.
 ")
 DOC_SECTION (FILE-OPERATORS)
+DEFUN (pf_dir::read-file,  filename &optional varname,
+" Read in the contents of the file named by FILENAME and return them.
+ If the optional variable <var varname> is given, it is the name of
+ a binary variable to receive the contents of the file.  In that case,
+ the word \"true\" is returned if the file was successfully read, or the
+ empty string otherwise.
+
+ <example>
+ <dir::read-file /tmp/foo.gif gifdata> --true
+ <set-var hosts[]=<dir::read-file /etc/hosts>>
+ </example>
+")
+DOC_SECTION (FILE-OPERATORS)
+DEFUN (pf_dir::write-file,  filename varname,
+" Write the contents of the variable named by VARNAME to the file
+ named by FILENAME.  Returns \"true\" if the file was written, and
+ the empty string otherwise.
+")
+DOC_SECTION (FILE-OPERATORS)
 DEFUN (pf_directory-p,  var,
 " Treat the argument <var var> as the name of a variable which contains
  an alist produced by <funref file-operators get-file-properties> or
@@ -121,15 +183,27 @@ DEFUN (pf_directory?,  pathname,
 " Return \"true\" if <var pathname> is a directory, or \"\" if not.
 ")
 DOC_SECTION (LANGUAGE-OPERATORS)
-DEFUN (pf_dump-all-packages, ,
-" Call <funref language-operators dump-package> on every defined package.
+DEFUN (pf_dump-alist,  #alist &key recurse?,
+" Dump out the contents of <var alist> in the same fashion as 
+ <funref language-operators dump-package> would.
 ")
 DOC_SECTION (LANGUAGE-OPERATORS)
-DEFUN (pf_dump-package,  pack &optional recurse?,
-" Dump the contents of <var pack> in a format suitable for including in
- <code>HTML</code> output.  If second argument <var RECURSE?> is
- non-empty, then recursively dump the contents of any alists
- found as values in <var pack>.
+DEFUN (pf_dump-all-packages,  &key recurse?,
+" Call <funref language-operators dump-package> on every defined package.
+ If the keyword argument <var recurse?> is supplied, it indicates
+ that any association lists found as values in the package
+ variables should be dumped out as well.
+")
+DOC_SECTION (LANGUAGE-OPERATORS)
+DEFUN (pf_dump-package,  &key recurse? text-mode? &rest packages[],
+" Dump the contents of the specified <var packages> in a format
+suitable for including in <code>HTML</code> output.  If the
+keyword argument <var RECURSE?> is non-empty, then recursively
+dump the contents of any alists found as values in <var pack>.
+ If the keyword argument <var text-mode?> is non-empty, then
+ dump the contents of the packages in a format suitable for
+ placing within a &lt;PRE&gt; ... &lt;/PRE&gt; construct.  This
+ happens automatically when running under <b>mdb</b>.
 ")
 DOC_SECTION (NETWORK-APPENDIX)
 DEFUN (pf_e-mail::sendmail,  from to[] cc[] bcc[] headers[] message,
@@ -189,8 +263,51 @@ DEFMACRO (pf_form-to-database,  database &key dbname key redirect-to action-name
  Defaults to \"Submit Form\".
 </ul>
 ")
+DOC_SECTION (MISCELLANEOUS-TAGS)
+DEFUN (pf_forms::multiple-instance-resolve,  session-pack dest-pack,
+" DOC-SECTION (HTML-HELPERS)
+ For every instance of a form in SESSION-PACK, append the
+ elements in order to arrays in DEST-PACK.  Thus, if your
+ form contained \"name\" and \"age\" input fields, and the user
+ created multiple instances of this form, the output of
+ calling (forms::multiple-instance-resolve mypack results)
+ would be:
+ RESULTS::NAME[0] --> Brian
+ RESULTS::NAME[1] --> John
+ ...
+ RESULTS::AGE[0] --> 38  (i.e., Brian's age)
+ RESULTS::AGE[1] --> 37  (i.e., John's age).
+
+ You have to call FORMS::MULTIPLE-INSTANCE-RESOLVE for every form
+ in your page which used FORMS::MULTIPLE-INSTANCES.  Since this
+ requires the use of the session database, you can call
+ FORMS::MULTIPLE-INSTANCE-RESOLVE from any page at any time to read
+ the contents of the session data, and return the information.
+")
+DOC_SECTION (HTML-HELPERS)
+DEFMACRO (pf_forms::multiple-instances,  session-pack &key modify-hook create-hook remove-hook method action enctype &body &unevalled the-form-body,
+" Creates a repeating form, allowing multiple entries of data sets.
+ The body of this macro is exactly whatever you might put in a
+ standard FORM body.  What is needed here is examples, and there
+ really isn't space for any.  Please see the file called
+ multiple-instances.mhtml in the distribution for examples.
+")
+DOC_SECTION (HTML-HELPERS)
+DEFUN (pf_forms::standard-buttons,  &optional forms-index,
+" Display the standard buttons for input when performing
+ <funref HTML-HELPERS forms::multiple-instances>.
+ If there are no current instances of the form, this produces
+ a language sensitive submit button with the name of \"form-create\" --
+ if the instance that this function is called in already has values,
+ then two submit buttons are produced, with names \"form-modify\" and
+ \"form-accept\".
+
+ If you would like to control the placement of these buttons, then
+ you should set the variable <var forms::no-submits>.  In this case,
+ you are responsible for calling forms::standard-buttons yourself.
+")
 DOC_SECTION (NETWORK-APPENDIX)
-DEFUN (pf_http::get-document,  host port path &key strip-headers,
+DEFUN (pf_http::get-document,  host port path &key strip-headers var &rest headers[],
 " Get an HTTP document specified by <var host>, <var port> and <var path>.
 
  If a redirection is necessary in order to get the final
@@ -205,6 +322,18 @@ DEFUN (pf_http::get-document,  host port path &key strip-headers,
  If <var strip-headers> is given, the HTTP protocol headers are
  stripped from the document before it is returned.
 
+ If the keyword argument <var var> is supplied, it is the
+ name of variable that should receive the contents of the HTTP
+ document verbatim -- by necessity, this variable is binary.
+ You can use the <var content-length> function to find out the
+ length of the retrieved document.
+
+ Finally, any extra headers that you would like to pass along may
+ be written into the call, perhaps to set a cookie for example.
+
+ <example>
+ <http::include http://www.metahtml.com/ \"Cookie: SID=829389834\">
+ </example>
 ")
 DOC_SECTION (NETWORK-APPENDIX)
 DEFUN (pf_http::host-part,  url,
@@ -214,9 +343,18 @@ DEFUN (pf_http::host-part,  url,
  </complete-example>
 ")
 DOC_SECTION (NETWORK-APPENDIX)
-DEFUN (pf_http::include,  url,
+DEFUN (pf_http::include,  url &key varname &rest extra-headers[],
 " Include the contents of the Web document referenced by <var url>
- in the page.
+ in the page.  If <var varname> is supplied, it is the name of a
+ binary variable which will receive the page contents.  Otherwise,
+ the document is simply returned.
+
+ <var extra-headers> can be supplied, perhaps to pass a cookie to
+ the server:
+
+ <example>
+ <http::include http://www.foo.com/cgi-bin/foo.exe \"Cookie: SID=10\">
+ </example>
 ")
 DOC_SECTION (NETWORK-APPENDIX)
 DEFUN (pf_http::path-part,  url,
@@ -235,8 +373,51 @@ DEFUN (pf_http::port-part,  url,
  <http::port-part http://www.metahtml.com:8080/page.mhtml>
  </complete-example>
 ")
+DOC_SECTION (NETWORK-APPENDIX)
+DEFUN (pf_http::post,  url pack &key user-agent &rest http-headers,
+" POST the variables and values in <var pack> to the server and page
+ specified by <var url>.  If the keyword argument <var user-agent>
+ is supplied, it overides the default value, which is either the
+ value of <var env::http_user_agent>, or the string
+ <b><get-var-once http::default-user-agent></b>.
+")
+DOC_SECTION (MISCELLANEOUS-TAGS)
+DEFUN (pf_lang,  &key en de fr it es,
+" Provides an interface for selecting one of several phrases
+ in different languages, based on the value of AUTHDATA::LANGUAGE.
+
+ Use in the following situations:
+ <example code>
+ <input type=submit name=action value=<lang en=Now de=Jetzt>>
+ </example>
+
+")
+DOC_SECTION (MISCELLANEOUS-TAGS)
+DEFUN (pf_lang-inform,  &key en de fr it es &rest info,
+" This function uses the value of authdata::language to select a
+ language, and then calls <var inform> with <info>.
+")
+DOC_SECTION (MISCELLANEOUS-TAGS)
+DEFUN (pf_lang-message,  &key en de fr it es &rest message,
+" This function uses the value of authdata::language to select a
+ language, and then calls <var message> with that choice.
+")
 DOC_SECTION (HTML-HELPERS)
-DEFUN (pf_locale::select-common-country,  postname sitename,
+DEFUN (pf_locale::iso-country-name-to-abbrev2,  country-name,
+" Returns the two-letter abbreviation of the ISO-3166 country name
+ passsed in <var country-name>.  If <var country name> isn't an
+ ISO-3166 country name, then it is simply returned as is.
+")
+DOC_SECTION (HTML-HELPERS)
+DEFUN (pf_locale::iso-country-name-to-abbrev3,  country-name,
+" Returns the three-letter abbreviation of the ISO-3166 country name
+ passsed in <var country-name>.  If there isn't a three-letter
+ abbreviation, but there is a two-letter one, then that is returned.
+ If <var country name> isn't an ISO-3166 country name, then it is
+ simply returned as is.
+")
+DOC_SECTION (HTML-HELPERS)
+DEFUN (pf_locale::select-common-country,  postname &optional sitename,
 " Produce a selection widget suitable for placing in a form which
  allows the user to select a country from a pull-down menu.
  <var postname> is the name of the variable which should be posted
@@ -248,7 +429,7 @@ DEFUN (pf_locale::select-common-country,  postname sitename,
  only in the number of countries that are displayed.
 ")
 DOC_SECTION (HTML-HELPERS)
-DEFUN (pf_locale::select-country,  postname sitename,
+DEFUN (pf_locale::select-country,  postname &optional sitename,
 " Produce a selection widget suitable for placing in a form which
  allows the user to select a country from a pull-down menu.
  <var postname> is the name of the variable which should be posted
@@ -268,14 +449,67 @@ DEFUN (pf_locale::select-from-country,  postname sitename country-array,
  qualified with its package name (i.e., DEFAULT::FOO, or SITE::COUNTRY).
 ")
 DOC_SECTION (HTML-HELPERS)
+DEFUN (pf_locale::select-iso-country,  postname &optional sitename,
+" Produce a selection widget suitable for placing in a form which
+ allows the user to select a country from a pull-down menu which
+ includes all of the ISO-3166 country names.
+ <var postname> is the name of the variable which should be posted
+ when the form is submitted.
+ <var sitename> is the name of another variable which can contain the
+ currently selected country -- this variable must be fully
+ qualified with its package name (i.e., DEFAULT::FOO, or SITE::COUNTRY).
+")
+DOC_SECTION (HTML-HELPERS)
+DEFUN (pf_locale::select-iso-country-abbrev2,  postname &optional sitename,
+" Produce a selection widget suitable for placing in a form which
+ allows the user to select an ISO-3166 country name from a pull-down menu.
+ <var postname> is the name of the variable which should be posted
+ when the form is submitted.  The long name of the country is displayed,
+ but the short, two-letter version of the country is returned.
+ <var sitename> is the name of another variable which can contain the
+ currently selected country.
+")
+DOC_SECTION (HTML-HELPERS)
+DEFUN (pf_locale::select-iso-country-abbrev3,  postname &optional sitename,
+" Produce a selection widget suitable for placing in a form which
+ allows the user to select an ISO-3166 country name from a pull-down menu.
+ <var postname> is the name of the variable which should be posted
+ when the form is submitted.  The long name of the country is displayed,
+ but the short, three-letter version of the country is returned.
+ <var sitename> is the name of another variable which can contain the
+ currently selected country.
+")
+DOC_SECTION (HTML-HELPERS)
+DEFUN (pf_locale::select-province,  postname sitename,
+" Produce a selection widget suitable for placing in a form which
+ allows the user to select a province name from a pull-down menu.
+ <var postname> is the name of the variable which should be posted
+ when the form is submitted.  The long name of the province is displayed,
+ but the short, two- or three-letter version of the province is returned.
+ <var sitename> is the name of another variable which can contain the
+ currently selected province.
+")
+DOC_SECTION (HTML-HELPERS)
 DEFUN (pf_locale::select-state,  postname sitename,
 " Produce a selection widget suitable for placing in a form which
  allows the user to select a US state name from a pull-down menu.
  <var postname> is the name of the variable which should be posted
- when the form is submitted.
+ when the form is submitted.  The long name of the state is displayed,
+ but the short, two-letter version of the state is returned.
  <var sitename> is the name of another variable which can contain the
  currently selected state.
 ")
+DOC_SECTION (HTML-HELPERS)
+DEFUN (pf_locale::select-state-or-province,  postname sitename,
+" Produce a selection widget suitable for placing in a form which
+ allows the user to select a US state or province name from a
+ pull-down menu.  <var postname> is the name of the variable
+ which should be posted when the form is submitted.  The long
+ name of the state or province is displayed, but the short, two-
+ or three-letter version of the state or province is returned.
+ <var sitename> is the name of another variable which can contain the
+ currently selected state or province.
+")
 DOC_SECTION (MISCELLANEOUS-TAGS)
 DEFUN (pf_mailable-address,  addr,
 " Return \"true\" if ADDR appears to be a correctly formatted E-mail address.
@@ -292,13 +526,6 @@ DEFUN (pf_make-directories,  path &optional mode-string,
 " Create all of the directories in <var path> (a fully qualified path to
  a directory) if they are not already present.
 ")
-DOC_SECTION (MISCELLANEOUS-TAGS)
-DEFUN (pf_make-identifier,  limit,
-" Create an identifier that is <var limit> characters in length.
- The identifier characters are taken from the array MI::ALPHABET.
- MI::ALPHABET defaults to uppercase letters and numbers excluding
- \"I\", \"O\", zero, and one.
-")
 DOC_SECTION (PACKAGES)
 DEFUN (pf_make-innocuous,  &rest packages[],
 " Causes each variable in the specified <var packages> to have
@@ -402,15 +629,24 @@ DEFUN (pf_percent,  num total,
  <percent 20 200> --> 10.00
  </example>
 ")
+DOC_SECTION (LANGUAGE-OPERATORS)
+DEFUN (pf_primitive?,  name,
+" Returns \"true\" if <var name> is defined as a Meta-HTML primitive, or
+ the empty string otherwise.
+")
 DOC_SECTION (HTML-HELPERS)
-DEFUN (pf_select-option, ,
+DEFUN (pf_select-option,  varname display &optional value,
 " Use instead of the <example code><option></example> HTML tag within
  the select HTML tag.
 
- It assumes that the text you wish to display on the page is in
- fact the value you wish to return from the form when this option
- is selected.  Particularly suited to situations where it is 
- helpful to know what the currently 'SELECTED' option might be.
+ Creates a <i>option</i> item in the page in which <var varname>
+ is expected to contain the current value of the choice, or nothing,
+ if no choice has been made.  If you supply the optional <var value>
+ argument, that value is returned instead of <var display>.
+
+ If <var varname> contains <var value>, then this option has the
+ HTML <b>SELECTED</b> attribute turned on.
+
  Example:
  <example>
  <select name=airport>
@@ -425,28 +661,30 @@ DEFUN (pf_select-option, ,
 </example>
 ")
 DOC_SECTION (HTML-HELPERS)
-DEFUN (pf_select-option-value, ,
-" Use instead of the <example code><option></example> HTML tag
- within the `SELECT' HTML tag, when the text you wish to display
- is actually different from that you wish to post back to the
- server.  Also suited to helping the user see what the currently
- 'SELECTED' option might be.
- 
+DEFUN (pf_select-option-value,  varname display &optional value,
+" Use instead of the <example code><option></example> HTML tag within
+ the select HTML tag.
+
+ Creates a <i>option</i> item in the page in which <var varname>
+ is expected to contain the current value of the choice, or nothing,
+ if no choice has been made.  If you supply the optional <var value>
+ argument, that value is returned instead of <var display>.
+
+ If <var varname> contains <var value>, then this option has the
+ HTML <b>SELECTED</b> attribute turned on.
+
  Example:
  <example>
  <select name=airport>
-   <select-option-value airport ALT Atlanta>
-   <select-option-value airport BTN Boston>
-   <select-option-value airport CHI Chicago>
-   <select-option-value airport LAS \"Los Angeles\">
-   <select-option-value airport NY  \"New York\">
-   <select-option-value airport MI  Miami>
-   <select-option-value airport SFO \"San Francisco\">
+   <select-option airport Atlanta>
+   <select-option airport Boston>
+   <select-option airport Chicago>
+   <select-option airport \"Los Angeles\">
+   <select-option airport \"New York\">
+   <select-option airport Miami>
+   <select-option airport \"San Francisco\">
  </select>
- </example>
- San Francisco is displayed on the page but SFO is actually
- what is returned as the selected options value.
-
+</example>
 ")
 DOC_SECTION (PACKAGES)
 DEFUN (pf_session-destroy-package,  package,
@@ -462,6 +700,19 @@ DEFUN (pf_session::initialize,  &optional timeout timeout-page,
  If the variable <code>SESSION::INHIBIT-SESSION</code> is
  non-empty, then no session processing is done.
 
+ If the session has timed out, (detected by the browser passing in a SID
+ which doesn't exist in the session database), then a new session is
+ created using that SID, and the variable <var session::restarted> is
+ set to \"true\".
+
+ If a new session had to be created, either because the browser had
+ a SID which was timed out, or because the browser didn't have a SID,
+ then the variable <var session::new-arrival> is set to \"true\".
+
+ If you supply <var timeout-page>, and the session was timed out,
+ then the browser is redirected to <var timeout-page>, and the
+ session is not instantiated in the session database (although the
+ aforementioned variables are still set).
 ")
 DOC_SECTION (SESSION-OPERATORS)
 DEFUN (pf_session::reset-cookie, ,
@@ -504,10 +755,26 @@ DEFUN (pf_sql::affected-rows,  cursor,
 ")
 DOC_SECTION (GENERIC-SQL-INTERFACE)
 DEFUN (pf_sql::cursor-get-column,  cursor column-number,
-" Called with a cursor which is the result from a call to
- <funref generic-sql-interface sql::database-exec-query>,
+" Called with a <var cursor> (the result from a call to
+ <funref generic-sql-interface sql::database-exec-query>),
+ which has been filled in by at least one call to
+ <funref generic-sql-interface sql::database-next-record>,
  this function returns the data from the current result set,
- in the column numbered <var column-number> (starting at column 0). 
+ in the column numbered <var column-number> (starting at column 0).
+
+ If there is no result set, or if the result set is empty, the
+ empty string is returned.
+
+ <example>
+ <sql::with-open-database db dsn=\"host=localhost;database=admin\">
+   <set-var query=\"select * from users\">
+    <sql::database-exec-query db <get-var query> cursor=row>
+    <set-var any? = <sql::database-next-record row>>
+    <when <get-var any?>>
+      Result: <msql::cursor-get-column row 0>
+    </when>
+ </sql::with-open-database>
+ </example>
 ")
 DOC_SECTION (GENERIC-SQL-INTERFACE)
 DEFUN (pf_sql::database-column-info,  dbvar table fieldname,
@@ -747,11 +1014,12 @@ DEFUN (pf_sql::database-save-package,  dbvar key package &key table=tablename ke
  support NULL values on that column, the operation will fail.
 ")
 DOC_SECTION (GENERIC-SQL-INTERFACE)
-DEFUN (pf_sql::database-save-record,  dbvar key table keyname &optional var... &key table=tablename keyname=fieldname,
+DEFUN (pf_sql::database-save-record,  dbvar key &optional var... &key table=tablename key=fieldname,
 " Write a record to the open SQL database connection referenced by
  <var dbvar>, using <var key> as the value of the primary key, on
- the table <var table>.  <var KEYNAME> is the SQL column name of
- the column that you are specifying as the primary key.
+ the table <var table>.  The keyword argument <var KEY> is the
+ SQL column name of the column that you are specifying as the
+ primary key.
 
  For each <var var>, treat the name of <var var> as a column
  name, and set that column's value to the value stored within
@@ -786,8 +1054,8 @@ DEFUN (pf_sql::database-set-options,  dbvar &key option-name=value,
  The character which is used to escape single quotes in strings.
  <li> <b>SQL-TRUNCATE-COLUMNS</b>:
  When set to a non-empty string, queries composed by
- <funref generic-sql-interface database-save-record> and
- <funref generic-sql-interface database-save-package> will have
+ <funref generic-sql-interface sql::database-save-record> and
+ <funref generic-sql-interface sql::database-save-package> will have
  character string field data automatically truncated to the maximum
  column width of the field being stored into, before the query is
  ever sent to the SQL server.
@@ -859,6 +1127,11 @@ DEFUN (pf_sql::host-databases,  &optional host,
  <code>odbc</code> databases always returns an empty array.
 ")
 DOC_SECTION (GENERIC-SQL-INTERFACE)
+DEFUN (pf_sql::number-of-rows,  cursor,
+" Return the number of rows SELECTed by the last SQL query execution,
+ associated with the database referenced indirectly by <var cursor>.
+")
+DOC_SECTION (GENERIC-SQL-INTERFACE)
 DEFUN (pf_sql::query-get-column, ,
 " Undocumented.
 ")
@@ -868,6 +1141,43 @@ DEFUN (pf_sql::recent-query, ,
  SQL database.  Can be useful in debugging.
 ")
 DOC_SECTION (GENERIC-SQL-INTERFACE)
+DEFUN (pf_sql::search-form-params,  alist &key dsn table key editurl defcol title,
+"
+ Get or set the standard search form parameters.
+ Always returns an association list of the current search form
+ parameters, and can optionally set them to new values.  In
+ typical usage, this function might be called as follows:
+
+ <example>
+ <set-var orig-params =
+    <sql::search-form-params DSN=<mydsn> table=CLIENTS>>
+ <sql::standard-search-form posted>
+ <set-var ignore = <sql::search-form-params <get-var-once orig-params>>>
+ </example>
+
+
+ We have provided a simple but useful generic search form for use in
+ your Meta-HTML pages.  Calling
+ <funref generic-sql-interface sql::standard-search-form> both produces
+ the search form, and the results.
+
+ You should set the following variables before calling
+ <var sql::standard-search-form>:
+
+ <ul>
+ <li> sql::search-dsn -- The DSN string for the database.
+ <li> sql::search-dbtable -- The name of the table in the database to search.
+ <li> sql::search-keyname -- The name of the key field in the database.
+ <li> sql::search-editurl -- The url of a single record editor.
+ <li> sql::search-defcol -- The default column to display in the list.
+ <li> sql::search-title -- The title to display in the input form.
+ </ul>
+
+ Optionally, you can specify columns that you would like not to be
+ offered as columns that may be searched for, in the array
+ sql::search-ignore-cols[].
+")
+DOC_SECTION (GENERIC-SQL-INTERFACE)
 DEFUN (pf_sql::set-database-type,  type,
 " Create a command set called SQL::... which contains all of the functions
  normally used for accessing ODBC, mySQL, or mSQL databases.  This allows
@@ -893,6 +1203,68 @@ DEFUN (pf_sql::sql-transact, ,
 " Currently unimplemented.
 ")
 DOC_SECTION (GENERIC-SQL-INTERFACE)
+DEFUN (pf_sql::standard-search-form,  package &key fsize href title,
+" Display a complete form suitable for searching for a record from
+ the database and table specified in sql::search-dsn and
+ sql::search-dbtable.
+
+ Optional keyword argument HREF specifies the destination page --
+ it defaults to the current page if not supplied.
+ Optional keyword FSIZE is the size of the font to use in the form --
+ this has the effect of growing or shrinking the form.
+
+ When this form is posted to, it gathers all of the
+ columns which match the specified search criteria, and then
+ displayes the results in a table, linking each to the
+ sql::search-editurl.  When only one result is found, redirects
+ to the page specified in search::editurl, providing a message in
+ the session, displayable with <funref html-helpers show-message>,
+ which explains why.
+")
+DOC_SECTION (GENERIC-SQL-INTERFACE)
+DEFUN (pf_sql::standard-search-handler,  package &key fsize href,
+" Perform a database search on the search table using the criteria
+ in PACKAGE.  Variables in PACKAGE must include:
+
+ <ol>
+ <li> csf-search   (simply must be set)
+ <li> csf-column   (the name of the column to search on)
+ <li> csf-operator (either \"LIKE\" \"NOT LIKE\" or \"=\")
+ <li> csf-value    (the text to search for)
+ </ol>
+
+ Produces a table with the contents of the specified column linked
+ with an href to sql::search-defurl?<cgi-encode sql::search-keyname>.
+
+ You can overide the information which is displayed for each record
+ that matches the search criteria by creating a function called
+ <b>sql::search-result-name</b>.  If this function exists, it should
+ take two arguments: the column name that was being searched for, and
+ an association list representing the data of the current record.
+
+ An example might be to always display the last and first name of
+ any search where the column name to be searched contained the
+ string \"name\":
+ <example>
+ <defun sql::search-result-name col alist>
+   <if <match <get-var-once col> \"name\" caseless=true>
+       <concat <alist-get-var alist last_name>
+               \", \"
+               <alist-get-var alist first_name>>
+     <alist-get-var alist <get-var-once col>>>
+ </defun>
+ </example>
+")
+DOC_SECTION (GENERIC-SQL-INTERFACE)
+DEFUN (pf_sql::translate-column,  col,
+" Return a pretty version of the column name specified in <var col>,
+ by translating it through the array <var sql::column-translations>.
+ Each member of that array is a column name followed by a colon,
+ followed by a pretty version of that column name.
+ If <var col> doesn't appear in the list of possible names, then
+ it is simply capitalized.
+")
+DOC_SECTION (GENERIC-SQL-INTERFACE)
 DEFMACRO (pf_sql::with-open-database,  dbvar &key dsn=dsn-string nolock=true,
 " Opens the database specified by DSN (database service name)
  string, and stores a referent to it in <var dbvar>.  If the
@@ -930,6 +1302,11 @@ DEFUN (pf_thispage, ,
 " The local web-based URL to this document, without the query string
  or path information.
 ")
+DOC_SECTION (LANGUAGE-OPERATORS)
+DEFUN (pf_user-function?,  name,
+" Returns \"true\" if <var name> is defined as a user function, or
+ the empty string otherwise.
+")
 DOC_SECTION (FILE-OPERATORS)
 DEFUN (pf_webpath-exists,  document-path,
 " Returns \"true\" if the specified <var document-path> exists with
