@@ -30,6 +30,11 @@
 
 #include "language.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 static void pf_with_open_stream (PFunArgs);
 static void pf_stream_put (PFunArgs);
 static void pf_stream_put_contents (PFunArgs);
@@ -74,6 +79,7 @@ typedef struct
   pid_t pid;		/* For PROG pipes, the pid of the child. */
   int mode;		/* Access mode. */
   int flags;		/* Special flags indicate changes in behaviour. */
+  int timeout;		/* Default value for IO timer. */
   int last_op;		/* The last operation done on this stream. */
 } Stream;
 
@@ -92,7 +98,8 @@ extern jmp_buf page_jmp_buffer;
 
 /* Install this stream, and return the index which is used to identify it. */
 static int
-add_open_stream (char *name, StreamType type, int fd, int mode, int flags)
+add_open_stream (char *name, StreamType type, int fd, int mode, int flags,
+		 int timeout)
 {
   Stream *stream = (Stream *)xmalloc (sizeof (Stream));
   int result = open_streams_index;
@@ -103,6 +110,7 @@ add_open_stream (char *name, StreamType type, int fd, int mode, int flags)
   stream->ifd = -1;
   stream->mode = mode;
   stream->flags = flags;
+  stream->timeout = timeout;
   stream->last_op = op_NONE;
 
   if (open_streams_index + 2 > open_streams_slots)
@@ -133,6 +141,13 @@ discard_stream (int stream_index)
 	  (stream->fd != fileno (stdout)) &&
 	  (stream->fd != fileno (stderr)))
 	close (stream->fd);
+      else
+	{
+	  if (stream->fd == fileno (stderr))
+	    fflush (stderr);
+	  else if (stream->fd == fileno (stdout))
+	    fflush (stdout);
+	}
 
       if (stream->ifd != -1)
 	{
@@ -165,19 +180,19 @@ stream_mode (char *modename)
 }
 
 static int
-stream_type (char *typename)
+stream_type (char *type_name)
 {
-  StreamType type = stream_FILE;
+  StreamType the_type = stream_FILE;
 
-  if (typename != (char *)NULL)
+  if (type_name != (char *)NULL)
     {
-      if (strcasecmp (typename, "tcp") == 0)
-	type = stream_TCP;
-      else if (strcasecmp (typename, "prog") == 0)
-	type = stream_PROG;
+      if (strcasecmp (type_name, "tcp") == 0)
+	the_type = stream_TCP;
+      else if (strcasecmp (type_name, "prog") == 0)
+	the_type = stream_PROG;
     }
 
-  return (type);
+  return (the_type);
 }
 
 static char **
@@ -245,8 +260,8 @@ int mhtml_stdin_fileno = 0;
 int mhtml_stdout_fileno = 1;
 int mhtml_stderr_fileno = 2;
 
-DEFMACRO (pf_with_open_stream,
-	  var name &key type=(FILE|TCP|PROG) mode=OPEN-MODE notimeout=true,
+DEFMACRO (pf_with_open_stream, var name
+	  &key type=(FILE|TCP|PROG) mode=OPEN-MODE timeout=(value|NEVER),
 "Create an environment in which <var var> is bound to the
 <i>indicator</i> of an open stream, named by <var name>.
 
@@ -292,7 +307,17 @@ underlying object.
   <li> <code>WRITE-CREATE</code>:<br>
   The stream is opened for writing only.  The underlying object is
 created fresh, even if it had already existed.
-</ul>")
+</ul>
+
+When one is opening, reading to, or writing from a network stream, the 
+amount of time it can take to finish the operation is indeterminate.  So,
+the keyword argument <var timeout=value> can be used to specify the maximum
+amount of time (in seconds) that these operations may take.  If the value
+is specified as \"never\", then these operations can take \"forever\".
+If the value isn't specified, then the operations calculate the maximum
+wait time dynamically, based upon the amount of information to be read
+or written.  Any other value specifies an absolute maximum number of
+seconds.")
 {
   char *varname = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *filename = mhtml_canonicalize_file_name_argument
@@ -303,12 +328,14 @@ created fresh, even if it had already existed.
 
   if ((!empty_string_p (varname)) && (!empty_string_p (filename)))
     {
-      char *modename = mhtml_evaluate_string (get_value (vars, "MODE"));
-      char *typename = mhtml_evaluate_string (get_value (vars, "TYPE"));
+      char *mode_name = mhtml_evaluate_string (get_value (vars, "MODE"));
+      char *type_name = mhtml_evaluate_string (get_value (vars, "TYPE"));
       char *notimeout = mhtml_evaluate_string (get_value (vars, "NOTIMEOUT"));
-      int mode = stream_mode (modename);
-      int type = stream_type (typename);
+      char *timeout_arg = mhtml_evaluate_string (get_value (vars, "TIMEOUT"));
+      int mode = stream_mode (mode_name);
+      int type = stream_type (type_name);
       int flags = notimeout ? stream_NOTIMEOUT : 0;
+      int timeout = 0;
       int stream_fd = -1;
 
       /* For handling PROG streams. */
@@ -316,10 +343,25 @@ created fresh, even if it had already existed.
       int stdout_pipe[2];
       pid_t child = -1;
 
-      xfree (modename);
-      xfree (typename);
+      xfree (mode_name);
+      xfree (type_name);
       xfree (notimeout);
 
+      if (!empty_string_p (timeout_arg))
+	{
+	  if (strcasecmp (timeout_arg, "never") == 0)
+	    {
+	      timeout = 99999;
+	      flags |= stream_NOTIMEOUT;
+	    }
+	  else
+	    timeout = atoi (timeout_arg);
+	}
+      else
+	timeout = 0;
+
+      xfree (timeout_arg);
+
       switch (type)
 	{
 	case stream_FILE:
@@ -408,7 +450,7 @@ created fresh, even if it had already existed.
 	  char stream_value[40];
 	  PAGE *body_code = page_copy_page (body);
 	  int page_value =
-	    add_open_stream (filename, type, stream_fd, mode, flags);
+	    add_open_stream (filename, type, stream_fd, mode, flags, timeout);
 
 	  sprintf (stream_value, "%d", page_value);
 	  pagefunc_set_variable (varname, stream_value);
@@ -516,7 +558,7 @@ operation_timed_out (int sig)
    deliver each 1k of data, and then cushion that with a 10 second
    grace period.  Gee, isn't the Internet Fantastic? */
 static void
-set_io_timer (int length)
+set_io_timer (Stream *stream, int length)
 {
   char *minimum_data_transfer_rate = 
     pagefunc_get_variable ("mhtml::minimum-data-transfer-rate");
@@ -528,8 +570,13 @@ set_io_timer (int length)
   if (divisor < 1)
     divisor = 512;
 
-  timeout = length / divisor;
-  timeout += 10;
+  if (stream->timeout)
+    timeout = stream->timeout;
+  else
+    {
+      timeout = length / divisor;
+      timeout += 10;
+    }
 
   operation_timed_out_p = 0;
   signal (SIGALRM, operation_timed_out);
@@ -545,7 +592,7 @@ stream_write (Stream *stream, int length, char *bytes)
   if ((stream->flags & stream_MUST_SEEK) && (stream->last_op != op_WRITE))
     lseek (stream->fd, 0l, 1);
 
-  set_io_timer (length);
+  set_io_timer (stream, length);
 
   if (setjmp (operation_jmp_buf))
     {
@@ -592,9 +639,9 @@ stream_read (Stream *stream, char char_to_stop_at)
     lseek (input_fd, 0l, 1);
 
   if (stream->flags & stream_NOTIMEOUT)
-    set_io_timer (999999);
+    set_io_timer (stream, 999999);
   else
-    set_io_timer (80);
+    set_io_timer (stream, 80);
 
   if (setjmp (operation_jmp_buf))
     {
@@ -722,10 +769,7 @@ stream_read_chunk (Stream *stream, int length)
 
   buffer = (char *)xmalloc (length + 1);
 
-  if (stream->flags & stream_NOTIMEOUT)
-    set_io_timer (999999);
-  else
-    set_io_timer (length);
+  set_io_timer (stream, length);
 
   if (setjmp (operation_jmp_buf))
     {
@@ -1066,3 +1110,7 @@ processing data.")
   if ((stream != (Stream *)NULL) && (stream->type != stream_CLOSED))
     shutdown (stream->fd, 2);
 }
+
+#if defined (__cplusplus)
+}
+#endif
