@@ -29,7 +29,20 @@
    93101  */
 
 #if !defined (_STREAMFUNCS_H_)
+#define _STREAMFUNCS_H_
+
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 extern int mhtml_stdin_fileno;
 extern int mhtml_stdout_fileno;
 extern int mhtml_stderr_fileno;
+
+#if defined (__cplusplus)
+}
+#endif
+
 #endif /* _STREAMFUNCS_H_ */
+
