@@ -37,6 +37,11 @@
 /*							    */
 /************************************************************/
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 static void pf_string_length (PFunArgs);
 static void pf_match (PFunArgs);
 static void pf_string_compare (PFunArgs);
@@ -49,12 +54,16 @@ static void pf_capitalize (PFunArgs);
 static void pf_word_wrap (PFunArgs);
 static void pf_pad (PFunArgs);
 static void pf_string_eq (PFunArgs);
+static void pf_string_neq (PFunArgs);
 static void pf_plain_text (PFunArgs);
 static void pf_base64decode (PFunArgs);
 static void pf_base64encode (PFunArgs);
 
 #if defined (HAVE_CRYPT)
-static void pf_unix_crypt (PFunArgs);
+#  if defined (HAVE_CRYPT_H)
+#    include <crypt.h>
+#  endif
+  static void pf_unix_crypt (PFunArgs);
 #endif
 
   /* Random string operations. */
@@ -72,6 +81,7 @@ static PFunDesc func_table[] =
   { "WORD-WRAP",	0, 0, pf_word_wrap },
   { "PAD",		0, 0, pf_pad },
   { "STRING-EQ",	0, 0, pf_string_eq },
+  { "STRING-NEQ",	0, 0, pf_string_neq },
   { "PLAIN-TEXT",	1, 0, pf_plain_text },
   { "BASE64DECODE",	0, 0, pf_base64decode },
   { "BASE64ENCODE",	0, 0, pf_base64encode },
@@ -105,7 +115,7 @@ DEFUN (pf_string_length, string,
 
 #define MAX_SUBEXPS 10
 DEFUN (pf_match, string regex
-       &key action=[delete | extract | report | startpos | endpos],
+       &key action=[delete | extract | report | startpos | endpos | length],
 "Matches <var regexp> against <var string>, and then performs the
 indicated <var action>.  The default for <var action> is \"report\".
 
@@ -160,7 +170,7 @@ Here are a few examples:
       char *string = mhtml_evaluate_string (_string);
       char *regex = mhtml_evaluate_string (_regex);
       int caseless = var_present_p (vars, "caseless");
-      char *action = "report";
+      char *action = (char *)NULL;
 
       if ((string != (char *)NULL) && (regex != (char *)NULL))
 	{
@@ -170,10 +180,16 @@ Here are a few examples:
 	  int slen = strlen (string);
 	  int matched;
 	  int so = 0, eo = 0, len = 0;
-	  char *temp = get_value (vars, "action");
+	  char *temp = mhtml_evaluate_string (get_value (vars, "action"));
 	  char *packname = mhtml_evaluate_string (get_value (vars, "package"));
 
-	  if (temp) action = temp;
+	  if (!empty_string_p (temp))
+	    action = temp;
+	  else
+	    {
+	      xfree (temp);
+	      action = strdup ("report");
+	    }
 
 	  regcomp (&re, regex, REG_EXTENDED | (caseless ? REG_ICASE : 0));
 
@@ -270,8 +286,9 @@ Here are a few examples:
 	  regfree (&re);
 	}
 
-      if (string) free (string);
-      if (regex) free (regex);
+      xfree (string);
+      xfree (regex);
+      xfree (action);
     }
 
   if (result)
@@ -381,29 +398,29 @@ subst_in_string_internal (char *contents, Package *vars, int debug_level)
 
       while (!done)
 	{
-	  char *this = get_positional_arg (vars, arg++);
-	  char *that = get_positional_arg (vars, arg++);
+	  char *this_string = get_positional_arg (vars, arg++);
+	  char *with_that = get_positional_arg (vars, arg++);
 
-	  if (this == (char *)NULL)
+	  if (this_string == (char *)NULL)
 	    done = 1;
 	  else
 	    {
-	      this = mhtml_evaluate_string (this);
-	      that = mhtml_evaluate_string (that);
+	      this_string = mhtml_evaluate_string (this_string);
+	      with_that = mhtml_evaluate_string (with_that);
 
 	      if (debug_level > 5)
 		page_debug
 		  ("<subst-in-xxx \"%s\" \"%s\" \"%s\">",
-		   contents, this, that ? that : "");
+		   contents, this_string, with_that ? with_that : "");
 
-	      if (this)
-		page_subst_in_page (temp, this, that);
+	      if (this_string != (char *)NULL)
+		page_subst_in_page (temp, this_string, with_that);
 
 	      if (debug_level > 5)
 		page_debug ("--> `%s'", temp->buffer ? temp->buffer : "");
 
-	      if (this) free (this);
-	      if (that) free (that);
+	      xfree (this_string);
+	      xfree (with_that);
 	    }
 	}
 
@@ -414,11 +431,11 @@ subst_in_string_internal (char *contents, Package *vars, int debug_level)
   return (result);
 }
 
-DEFUN (pf_subst_in_var, varname &optional this with-that,
-"Replaces all occurrences of <var this> with <var with-that> in the
-contents of the variable named <var varname>.  Both <var this> and
+DEFUN (pf_subst_in_var, varname &optional this-string with-that,
+"Replaces all occurrences of <var this-string> with <var with-that> in the
+contents of the variable named <var varname>.  Both <var this-string> and
 <var with-that> are evaluated before the replacement is done. <var
-this> can be any regular expression allowed by the POSIX extended
+this-string> can be any regular expression allowed by the POSIX extended
 regular expression matching.  This command can be useful when parsing
 the output of <funref osfuncs cgi-exec>.")
 {
@@ -447,6 +464,10 @@ DEFUN (pf_subst_in_string, string &rest regexp replacement,
 <var regexp> can be any regular expression allowed by POSIX extended
 regular expression matching.
 
+In the replacement string, a backslash followed by a number <var n> is
+replaced with the contents of the <var n>th subexpression from <var
+regexp>.
+
 <example>
 <set-var foo=\"This is a list\">
 <subst-in-string <get-var foo> \"is\" \"HELLO\">
@@ -480,9 +501,10 @@ lowercase.
 <downcase \"This is Written in Meta-HTML\">
 </complete-example>")
 {
-  unsigned char *value = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  unsigned char *value = (unsigned char *)
+    mhtml_evaluate_string (get_positional_arg (vars, 0));
 
-  if (value != (char *)NULL)
+  if (value != (unsigned char *)NULL)
     {
       register int i;
 
@@ -504,9 +526,10 @@ uppercase.
 <upcase \"This is Written in Meta-HTML\">
 </complete-example>")
 {
-  unsigned char *value = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  unsigned char *value = (unsigned char *)
+    mhtml_evaluate_string (get_positional_arg (vars, 0));
 
-  if (value != (char *)NULL)
+  if (value != (unsigned char *)NULL)
     {
       register int i;
 
@@ -531,9 +554,10 @@ lowercase depending on the surrounding characters.
 Also see <funref string-operators downcase>, and
 <funref string-operators upcase>.")
 {
-  unsigned char *value = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  unsigned char *value = (unsigned char *)
+    mhtml_evaluate_string (get_positional_arg (vars, 0));
 
-  if (value != (char *)NULL)
+  if (value != (unsigned char *)NULL)
     {
       register int i;
       int capnext = 1;
@@ -894,6 +918,45 @@ comparison.
   xfree (arg2);
 }
 
+DEFUN (pf_string_neq, string-1 string-2 &key caseless=true,
+ "Compare <var string1> to <var string2> and return the string
+<code>\"true\"</code> if they are NOT character-wise identical.
+
+The optional keyword argument <var caseless=true> indicates that no
+consideration should be given to the case of the characters during
+comparison.
+
+<example>
+<string-neq \"foo\" \"FOO\">               -->true
+<string-neq \"foo\" \"foo\">               -->
+<string-neq <upcase \"foo\"> \"FOO\">      -->
+<string-neq \"foo\" \"FOO\" caseless=true> -->
+</example>")
+{
+  char *arg1 = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  char *arg2 = mhtml_evaluate_string (get_positional_arg (vars, 1));
+  char *caseless = mhtml_evaluate_string (get_value (vars, "caseless"));
+  int caseless_p = 0;
+
+  if (!empty_string_p (caseless))
+    caseless_p++;
+
+  xfree (caseless);
+
+  if ((empty_string_p (arg1) && !empty_string_p (arg2)) ||
+      (empty_string_p (arg2) && !empty_string_p (arg1)) ||
+      ((arg1 && arg2) &&
+       (((!caseless_p) && (strcmp (arg1, arg2) != 0)) ||
+	((caseless_p) && (strcasecmp (arg1, arg2) != 0)))))
+    {
+      bprintf_insert (page, start, "true");
+      *newstart = start + 4;
+    }
+
+  xfree (arg1);
+  xfree (arg2);
+}
+
 /* Does modifications to the plain text in BODY.  Usually, this simply
    inserts paragraph breaks where they appear, and optionally operates
    on the first character of paragraphs.  The text starts with a <P>,
@@ -1056,19 +1119,19 @@ displays the base64 encoded version:
 {
   char *varname = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *result = (char *)NULL;
-  char *sl_arg = mhtml_evaluate_string
+  char *sl_argstring = mhtml_evaluate_string
     (get_one_of (vars, "shortlines", "short-lines", (char *)NULL));
   int shortlines = 0;
 
-  if (!empty_string_p (sl_arg))
+  if (!empty_string_p (sl_argstring))
     {
-      shortlines = atoi (sl_arg);
+      shortlines = atoi (sl_argstring);
       if (!shortlines) shortlines = 64;
 
       shortlines = ((shortlines + 3) / 4) * 4;
     }
 
-  xfree (sl_arg);
+  xfree (sl_argstring);
 
   if (!empty_string_p (varname))
     {
@@ -1260,3 +1323,7 @@ pf_unix_crypt (PFunArgs)
   xfree (saltarg);
 }
 #endif /* HAVE_CRYPT */
+
+#if defined (__cplusplus)
+}
+#endif
