@@ -31,6 +31,10 @@
 
 #include "language.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 /************************************************************/
 /*							    */
 /*		 Variable Manipulation Functions	    */
@@ -52,6 +56,7 @@ static void pf_symbol_info (PFunArgs);
 static void pf_copy_var (PFunArgs);
 static void pf_coerce_var (PFunArgs);
 static void pf_defvar (PFunArgs);
+static void pf_alist_p (PFunArgs);
 
 static PFunDesc func_table[] =
 {
@@ -70,6 +75,7 @@ static PFunDesc func_table[] =
   { "COPY-VAR",		0, 0, pf_copy_var },
   { "COERCE-VAR",	0, 0, pf_coerce_var },
   { "DEFVAR",		0, 0, pf_defvar },
+  { "ALIST?",		0, 0, pf_alist_p },
 
   { (char *)NULL,	0, 0, (PFunHandler *)NULL }
 };
@@ -709,7 +715,7 @@ Each <var to-var> becomes the repository of a copy of the information already st
 		      Symbol *copy = symbol_copy (source, temp);
 
 		      copy = symbol_rename (copy, dest->name);
-		      symbol_move (copy, dest->package);
+		      symbol_move (copy, (Package *)dest->package);
 		      symbol_destroy_package (temp);
 		    }
 		}
@@ -802,26 +808,26 @@ value:
 		      {
 			/* Make each line of the source (including blanks)
 			   be a single element in the destination array. */
-			register int i, start;
+			register int i, beg;
 			int dst_index  = 0, dst_slots = 0;
 			char **array = (char **)NULL;
 			char *string = source->values[0];
 
-			start = 0; i = 0;
-			while (string[start] != '\0')
+			beg = 0; i = 0;
+			while (string[beg] != '\0')
 			  {
 			    /* Find end of line, or end of data. */
-			    for (i = start;
+			    for (i = beg;
 				 ((string[i] != '\0') && (string[i] != '\n'));
 				 i++);
 
-			    if ((start == i) && (string[i] == '\0'))
+			    if ((beg == i) && (string[i] == '\0'))
 			      break;
 			    else
 			      {
-				int size = i - start;
+				int size = i - beg;
 				char *line = (char *)xmalloc (1 + size);
-				strncpy (line, string + start, size);
+				strncpy (line, string + beg, size);
 				line[size] = '\0';
 
 				if (dst_index + 2 > dst_slots)
@@ -832,7 +838,7 @@ value:
 				array[dst_index++] = line;
 				array[dst_index] = (char *)NULL;
 				if (string[i]) i++;
-				start = i;
+				beg = i;
 			      }
 			  }
 
@@ -947,3 +953,33 @@ to <var value>.
 
   xfree (name);
 }
+
+DEFUNX (pf_alist?, string,
+"Return \"t\" if <var string> is a representation of an association list.")
+
+static void
+pf_alist_p (PFunArgs)
+{
+  char *string = mhtml_evaluate_string (get_positional_arg (vars, 0));
+  WispObject *list = string ? wisp_from_string (string) : NIL;
+  int is_alist = 0;
+
+  if ((CONS_P (list)) &&
+      (list != NIL) &&
+      (CONS_P (CAR (list))) &&
+      (STRING_P (CAR (CAR (list)))))
+    is_alist++;
+
+  gc_wisp_objects ();
+  xfree (string);
+
+  if (is_alist)
+    {
+      bprintf_insert (page, start, "t");
+      *newstart = *newstart + 1;
+    }
+}
+
+#if defined (__cplusplus)
+}
+#endif
