@@ -53,6 +53,14 @@
 #include "http.h"
 #include "globals.h"
 #include "logging.h"
+#if defined (HAVE_CRYPT_H)
+#include <crypt.h>
+#endif
+
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 
 #define HAVE_CRYPT 1
 
@@ -451,3 +459,7 @@ static int
 password_match (char *clear, char *password) { return (0); }
 #endif /* !HAVE_CRYPT */
 
+
+#if defined (__cplusplus)
+}
+#endif
