@@ -28,5 +28,13 @@
    Santa Barbara, CA
    93101  */
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 extern int mhttpd_check_access (HTTP_RESULT *result);
 
+
+#if defined (__cplusplus)
+}
+#endif
