@@ -70,6 +70,11 @@
 #include "logging.h"
 #include "streamfuncs.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 #if !defined (errno)
 extern int errno;
 #endif
@@ -344,7 +349,7 @@ gobble_word (char *line, int *start)
 static HTTP_REQUEST *
 http_make_request (void)
 {
-  HTTP_REQUEST *request = xmalloc (sizeof (HTTP_REQUEST));
+  HTTP_REQUEST *request = (HTTP_REQUEST *)xmalloc (sizeof (HTTP_REQUEST));
 
   memset (request, 0, sizeof (HTTP_REQUEST));
 
@@ -541,10 +546,12 @@ mhttpd_find_page_for_result (HTTP_RESULT *result)
 			 result->spec->logical_path ?
 			 result->spec->logical_path :
 			 result->spec->requested_path);
-  pagefunc_set_variable ("mhttpd::result-reason",
-			 mhttpd_result_reason (result->result_code));
-  pagefunc_set_variable ("mhttpd::request-method",
-			 result->request->method);
+  pagefunc_set_variable
+    ("mhttpd::result-reason",
+     mhttpd_result_reason (result->result_code));
+  pagefunc_set_variable
+    ("mhttpd::request-method",
+     result->request ? result->request->method : "UNKNOWN");
 
   if (uf)
     value = mhtml_evaluate_string ("<mhttpd::default-document>");
@@ -864,20 +871,20 @@ mr_location (HTTP_RESULT *result, char *existing_location)
       int port = atoi (pagefunc_get_variable ("mhtml::server-port"));
       char *server_name = pagefunc_get_variable ("mhtml::server-name");
       char *extra = pagefunc_get_variable ("rewriter::engine-webpath");
-      char *new;
+      char *newloc;
 
       if (extra == (char *)NULL) extra = "";
 
-      new = (char *)xmalloc
+      newloc = (char *)xmalloc
 	(100 + strlen (extra) + strlen (existing_location));
 
-      sprintf (new, "%s://%s", protocol, server_name);
+      sprintf (newloc, "%s://%s", protocol, server_name);
       if (((strcmp (protocol, "http") == 0) && (port != 80)) ||
 	  ((strcmp (protocol, "https") == 0) && (port != 443)))
-	sprintf (new + strlen (new), ":%d", port);
-      strcat (new, extra);
-      strcat (new, existing_location);
-      existing_location = new;
+	sprintf (newloc + strlen (newloc), ":%d", port);
+      strcat (newloc, extra);
+      strcat (newloc, existing_location);
+      existing_location = newloc;
     }
   return (existing_location);
 }
@@ -1282,11 +1289,25 @@ mhtml_post_to_document (char *string)
       {
 	Package *default_pack = symbol_get_package (DEFAULT_PACKAGE_NAME);
 	Symbol **symbols = symbol_package_symbols ("POSTED");
-
+	    
 	if (symbols != (Symbol **)NULL)
 	  {
+	    /* If maxed out on server debugging, show the contents of the
+	       posted package here. */
+	    char *debug_p = pagefunc_get_variable ("mhtml::debug-post");
+	    int debug = 0;
+
+	    if (!empty_string_p (debug_p)) debug=1;
+
 	    for (i = 0; symbols[i] != (Symbol *)NULL; i++)
-	      symbol_copy (symbols[i], default_pack);
+	      {
+		if (debug)
+		  mhttpd_log
+		    (log_DEBUG, "POSTED::%s = %s", symbols[i]->name,
+		     symbols[i]->values[0] ? symbols[i]->values[0] : "");
+
+		symbol_copy (symbols[i], default_pack);
+	      }
 	  }
       }
     }
@@ -1868,8 +1889,12 @@ mhttpd_metahtml_engine (HTTP_RESULT *result)
     result->result_code = res_NOT_FOUND;
   else
     {
+      char digits[40];
       char *enctype;
 
+      sprintf (digits, "%ld", page_most_recent_modification_time);
+      pagefunc_set_variable ("mhtml::last-modification-time", digits);
+      
       {
 	PAGE *p = page_create_page ();
 	bprintf (p, "<*parser*::push-file %s>", result->spec->physical_path);
@@ -2437,3 +2462,6 @@ http_unimplemented_handler (ReqFunArgs)
   return (result);
 }
 
+#if defined (__cplusplus)
+}
+#endif
