@@ -5,6 +5,11 @@
 #if !defined (_TCP_H_)
 #define _TCP_H_
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 /* Set to non-zero to override the standard connect() timeout with a
    one of length TIME_OUT. This is useful for large networks where
    otherwise wwwproxy could wait for long periods while trying to
@@ -17,5 +22,9 @@
 
 extern int tcp_to_host (char *host, char *service);
 extern unsigned char *hostname_or_ip_to_address (char *hostname_spec);
-#endif /* _TCP_H_ */
 
+#if defined (__cplusplus)
+}
+#endif
+
+#endif /* _TCP_H_ */
