@@ -36,6 +36,10 @@
 #include "wisp.h"
 #include <xmalloc/xmalloc.h>
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 static void initialize_string_buffer (void);
 static void print_object_to_string_buffer (WispObject *object);
 static void print_string_to_string_buffer (char *string);
@@ -89,6 +93,27 @@ obarray_del (WispObject *object)
       }
 }
 
+void
+gc_wisp_free (WispObject *object)
+{
+  switch (object->type)
+    {
+    case LT_string:
+      free (object->val.string);
+      free (object);
+      break;
+
+    case LT_number:
+    case LT_cons:
+    case LT_symbol:
+      free (object);
+      break;
+
+    case LT_nil:
+      break;
+    }
+}
+
 void
 gc_wisp_objects ()
 {
@@ -99,22 +124,7 @@ gc_wisp_objects ()
     {
       object = obarray[i];
 
-      switch (object->type)
-	{
-	case LT_string:
-	  free (object->val.string);
-	  free (object);
-	  break;
-
-	case LT_number:
-	case LT_cons:
-	case LT_symbol:
-	  free (object);
-	  break;
-
-	case LT_nil:
-	  break;
-	}
+      gc_wisp_free (object);
     }
   obarray_index = 0;
 }
@@ -133,6 +143,28 @@ allocate_object (void)
   return (object);
 }
 
+/* Steal OBJECT from the GC list so that it may persist.
+   The caller is then responsible for freeing the object
+   by calling gc_wisp_free () on it. */
+void
+gc_steal (WispObject *object)
+{
+  register int i;
+
+  for (i = 0; i < obarray_index; i++)
+    {
+      if (obarray[i] == object)
+	{
+	  for (; i < obarray_index; i++)
+	    obarray[i] = obarray[i + 1];
+
+	  obarray[i] = (WispObject *)NULL;
+	  obarray_index--;
+	  break;
+	}
+    }
+}
+  
 WispObject *
 make_string_object (char *string)
 {
@@ -900,3 +932,7 @@ wisp_length (WispObject *object)
     }
   return (result);
 }
+
+#if defined (__cplusplus)
+}
+#endif
