@@ -33,6 +33,13 @@
 #include "commands.h"
 #include "breakpoints.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+/* ../libmhtml/parser.c */
+extern char **mdb_pushed_function_names (void);
+
 /* readline/readline.c */
 extern char *readline (char *prompt);
 
@@ -74,7 +81,11 @@ static char *mdb_breakpoint (MDBArgs);
 static char *mdb_delete (MDBArgs);
 static char *mdb_cont (MDBArgs);
 static char *mdb_next (MDBArgs);
+static char *mdb_step (MDBArgs);
 static char *mdb_print (MDBArgs);
+static char *mdb_apropos (MDBArgs);
+static char *mdb_where (MDBArgs);
+static char *mdb_history (MDBArgs);
 
 MDBCommand mdb_command_table[] = {
   { "quit",	(char *)NULL,	mdb_quit,	"quit",
@@ -101,17 +112,29 @@ MDBCommand mdb_command_table[] = {
   { "cont",	"c",		mdb_cont,	"cont [COUNT]",
       "Continue from a breakpoint, perhaps skipping COUNT more" },
 
+  { "where",	"w",		mdb_where,	"where",
+      "Display the current stack state" },
+
   { "next",	"n",		mdb_next,	"next [COUNT]",
       "Executes COUNT more statements" },
 
+  { "step",	"s",		mdb_step,	"step [COUNT]",
+      "Steps into current expression" },
+
   { "load",	(char *)NULL,	mdb_load,	"load FILENAME",
       "Load FILENAME, making it the current file" },
 
   { "list",	"l",		mdb_list,	"list [FILENAME:]LINE",
       "List the page around LINE-NUMBER" },
 
-  { "info",	"i",		mdb_info,	"info [files|breakpoints]",
-      "Get information about files or breakpoints" },
+  { "info",	"i",		mdb_info,	"info [file|fun|breakpoint]",
+      "Get information about files, functions, or breakpoints" },
+
+  { "apropos",	(char *)NULL,	mdb_apropos,	"apropos STRING",
+      "List variables and functions that have STRING in their names" },
+
+  { "history",	(char *)NULL,	mdb_history,	"history [count]",
+      "Show COUNT lines from the history" },
 
   { (char *)NULL, (char *)NULL, (MFunction *)NULL, (char *)NULL, (char *)NULL }
 };
@@ -229,7 +252,7 @@ mdb_scan_for_defuns (MDB_File *file, PAGE *page)
 	      int start, end;
 	      char *fname = (char *)NULL;
 	      MDB_FunctionDef *fdef = (MDB_FunctionDef *)
-		xmalloc (sizeof (MDB_FunctionDef *));
+		xmalloc (sizeof (MDB_FunctionDef));
 
 	      while (!whitespace (page->buffer[i])) i++;
 	      while (whitespace (page->buffer[i]))
@@ -239,7 +262,8 @@ mdb_scan_for_defuns (MDB_File *file, PAGE *page)
 		  i++;
 		}
 	      start = i;
-	      while (!whitespace (page->buffer[i])) i++;
+	      while ((!whitespace (page->buffer[i])) &&
+		     (page->buffer[i] != '>')) i++;
 	      end = i;
 
 	      fname = (char *)xmalloc (1 + (end - start));
@@ -483,6 +507,13 @@ mdb_help (MDBArgs)
   return (result);
 }
 
+extern void add_history (char *line);
+static void mdb_parser_interrupt_callback (void);
+static void
+parser_interrupt (int sig)
+{
+  mhtml_parser_interrupted++;
+}
 
 static char *
 mdb_eval (MDBArgs)
@@ -511,6 +542,7 @@ mdb_eval (MDBArgs)
       else
 	{
 	  bprintf (page, "%s\n", line);
+	  add_history (line);
 	  free (line);
 	  syntax_complete = page_check_syntax (page);
 	}
@@ -520,7 +552,11 @@ mdb_eval (MDBArgs)
     {
       page_debug_clear ();
       page_syserr_clear ();
+      mhtml_parser_callback_function = mdb_parser_interrupt_callback;
+      signal (SIGINT, parser_interrupt);
       page_process_page (page);
+      signal (SIGINT, mdb_signal_restart);
+      mhtml_parser_callback_function = NULL;
 
       if (page)
 	{
@@ -531,13 +567,31 @@ mdb_eval (MDBArgs)
 
   if (page_debug_buffer ())
     {
-      fprintf (stderr, "[DEBUGGING-OUTPUT]: %s\n", page_debug_buffer ());
+      register int i;
+      char *x = page_debug_buffer ();
+
+      for (i = strlen (x) - 1; i > 0; i--)
+	if (x[i] == '\n')
+	  x[i] = '\0';
+	else
+	  break;
+
+      fprintf (stderr, "[DEBUGGING-OUTPUT\n%s]\n", x);
       page_debug_clear ();
     }
 
   if (page_syserr_buffer ())
     {
-      fprintf (stderr, "[SYSTEM-ERROR-OUTPUT]: %s\n", page_syserr_buffer ());
+      register int i;
+      char *x = page_syserr_buffer ();
+
+      for (i = strlen (x) - 1; i > 0; i--)
+	if (x[i] == '\n')
+	  x[i] = '\0';
+	else
+	  break;
+
+      fprintf (stderr, "[SYSTEM-ERROR-OUTPUT\n%s]\n", x);
       page_syserr_clear ();
     }
 
@@ -617,7 +671,7 @@ mdb_quit (MDBArgs)
   if (!answer || (strncasecmp (answer, "y", 1) == 0))
     {
       MDB_QuitFlag = 1;
-      mdb_throw_to_top_level ();
+      /* mdb_throw_to_top_level (); */
     }
 
   return ((char *)NULL);
@@ -877,7 +931,13 @@ mdb_list (MDBArgs)
       find_file_and_line_for_function (fname, &found, &line_number);
 
       if (found)
-	file = mdb_find_file (found);
+	{
+	  file = mdb_find_file (found);
+	  /* Make this function's definition line be at the top of the
+	     listing.  This assumes that we know how many lines appear in
+	     a listing. */
+	  line_number += 9;
+	}
     }
 
   if (!file)
@@ -928,16 +988,73 @@ mdb_file_info (void)
   return (result);
 }
 
+static char *
+strstrcase (char *haystack, char *needle)
+{
+  char *result = (char *)NULL;
+
+  if ((haystack != (char *)NULL) && (needle != (char *)NULL))
+    {
+      register int i;
+      int hl = strlen (haystack);
+      int nl = strlen (needle);
+
+      for (i = 0; i < (hl - nl); i++)
+	if (strncasecmp (haystack + i, needle, nl) == 0)
+	  {
+	    result = haystack + i;
+	    break;
+	  }
+    }
+
+  return (result);
+}
+
+static char *
+mdb_info_funs (char *text)
+{
+  register int i;
+  char *result;
+  BPRINTF_BUFFER *output = bprintf_create_buffer ();
+  MDB_File *last_file = (MDB_File *)NULL;
+
+  bprintf (output, "Functions loaded with `%s' in their names:\n", text);
+
+  for (i = 0; i < snarfed_index; i++)
+    {
+      if (strstrcase (snarfed_functions[i]->fname, text) == 0)
+	{
+	  if (snarfed_functions[i]->file != last_file)
+	    {
+	      last_file = snarfed_functions[i]->file;
+	      bprintf (output, "In file %s:\n",
+		       last_file->filename ? last_file->filename : "(prim)");
+	    }
+	  bprintf (output, "    %s\n", snarfed_functions[i]->fname);
+	}
+    }
+
+  result = output->buffer;
+  free (output);
+  return (result);
+}
+
 static char *
 mdb_info (MDBArgs)
 {
   char *result;
 
-  if ((strcasecmp (line, "files") == 0) ||
-      (!*line))
+  if ((strncasecmp (line, "fi", 2) == 0) || (!*line))
     result = mdb_file_info ();
-  else if (strncasecmp (line, "bre", 3) == 0)
+  else if (strncasecmp (line, "b", 1) == 0)
     result = mdb_breakpoint_info ();
+  else if (strncasecmp (line, "fu", 2) == 0)
+    {
+      register int i = 0;
+      while ((line[i] != '\0') && (!whitespace (line[i]))) i++;
+      while (whitespace (line[i])) i++;
+      result = mdb_info_funs (line + i);
+    }
   else
     result = make_error ("Unrecognized argument to %s, `%s'.", name, line);
 
@@ -1073,6 +1190,54 @@ mdb_next (MDBArgs)
   return (result);
 }
 
+/* Step into the running expression. */
+static char *
+mdb_step (MDBArgs)
+{
+  int count = atoi ((line && *line) ? line : "1");
+  char *result = (char *)NULL;
+
+  if (mdb_loop_level > 1)
+    {
+      if (!mdb_current_bp)
+	result = make_message ("Can't do STEP from this type of breakpoint!");
+      else
+	{
+	  mdb_set_step_breakpoint (mdb_current_bp);
+	  MDB_ContFlag += count;
+	}
+    }
+  return (result);
+}
+
+/* Where are we in the stack? */
+static char *
+mdb_where (MDBArgs)
+{
+  char **stack = mdb_pushed_function_names ();
+  char *result = (char *)NULL;
+
+  if ((stack == (char **)NULL) || (stack[0] == (char *)NULL))
+    result = make_message ("Not running any function!");
+  else
+    {
+      register int i;
+      BPRINTF_BUFFER *result_buff = bprintf_create_buffer ();
+
+      for (i = 0; stack[i] != (char *)NULL; i++)
+	{
+	  /* We don't have to print the last element, since it is
+	     always *MDB-Break*. */
+	  if (stack[i + 1] != (char *)NULL)
+	    bprintf (result_buff, "#%03d  %s\n", i, stack[i]);
+	}
+
+      result = result_buff->buffer;
+      free (result_buff);
+    }
+  return (result);
+}
+      
 /* Remove a breakpoint. */
 static char *
 mdb_delete (MDBArgs)
@@ -1205,6 +1370,18 @@ mdb_callback (PFunArgs, PFunDesc *desc, UserFunction *uf, char *open_body)
   return (1);
 }
 
+static void
+mdb_parser_interrupt_callback (void)
+{
+  printf ("Interrupt!\n");
+  mdb_loop ();
+
+  /* If the user has quit, under NO CIRCUMSTANCES should we
+     continue to execute the code. */
+  if (MDB_QuitFlag)
+    mdb_throw_to_top_level ();
+}
+
 static char *
 mdb_breakpoint (MDBArgs)
 {
@@ -1280,6 +1457,51 @@ mdb_breakpoint (MDBArgs)
   return (result);
 }
 
+static char *
+mdb_apropos (MDBArgs)
+{
+  BPRINTF_BUFFER *buffer = bprintf_create_buffer ();
+  char *result;
+
+  bprintf (buffer, "Symbols matching `%s'\n", line ? line : "");
+
+  if (line != (char *)NULL)
+    {
+      regex_t re;
+      regmatch_t offsets[2];
+
+      regcomp (&re, line, REG_EXTENDED | REG_ICASE);
+
+      if (AllPackages)
+	{
+	  register int pi;
+	  Package *pack;
+
+	  for (pi = 0; (pack = AllPackages[pi]) != (Package *)NULL; pi++)
+	    {
+	      /* Operate on global packages only. */
+	      if (pack->name != (char *)NULL)
+		{
+		  register int i;
+		  Symbol **syms = symbols_of_package (pack);
+
+		  for (i = 0; (syms && syms[i]); i++)
+		    {
+		      char *symname = syms[i]->name;
+
+		      if (regexec (&re, symname, 1, offsets, 0) == 0)
+			bprintf (buffer, "%s::%s\n", pack->name, symname);
+		    }
+		}
+	    }
+	}
+      regfree (&re);
+    }
+  result = buffer->buffer;
+  free (buffer);
+  return (result);
+}
+
 #if defined (MDB_EDITOR_COMMAND)
 static char *
 mdb_edit (MDBArgs)
@@ -1397,3 +1619,64 @@ pf_include_handler (PFunArgs)
   include_recursive_calls--;
 }
 
+#include "readline/history.h"
+
+/* Accessors for HIST_ENTRY lists that are called HLIST. */
+#define histline(i) (hlist[(i)]->line)
+#define histdata(i) (hlist[(i)]->data)
+static char *
+mdb_history (MDBArgs)
+{
+  register int i;
+  int limited, limit;
+  HIST_ENTRY **hlist;
+  BPRINTF_BUFFER *buffer = bprintf_create_buffer ();
+  char *result = (char *)NULL;
+
+  if (line && *line)
+    {
+      limited = 1;
+      limit = atoi (line);
+      if (limit == 0) limit = 20;
+    }
+  else
+    limited = limit = 0;
+
+  hlist = history_list ();
+
+  if (hlist)
+    {
+      for (i = 0;  hlist[i]; i++)
+	;
+
+      if (limit < 0)
+	limit = -limit;
+
+      if ((limited == 0)  || ((i -= limit) < 0))
+	i = 0;
+
+      while (hlist[i])
+	{
+	  bprintf (buffer, "%5d%c %s\n", i, histdata(i) ? '*' : ' ',
+		   histline(i));
+	  i++;
+	}
+    }
+
+  if (buffer->bindex)
+    {
+      result = buffer->buffer;
+      free (buffer);
+    }
+  else
+    {
+      result = (strdup ("No History\n"));
+      bprintf_free_buffer (buffer);
+    }
+
+  return (result);
+}
+
+#if defined (__cplusplus)
+}
+#endif
