@@ -30,12 +30,34 @@
 #include "language.h"
 #include "mdb.h"
 #include "commands.h"
+
 #if !defined (MHTML_SYSTEM_TYPE)
 #  define MHTML_SYSTEM_TYPE "Incorrectly Compiled"
 #endif
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
+typedef struct
+{
+  char *line;
+  char *data;
+} HIST_ENTRY;
+
 extern char *readline (char *prompt);
+extern int history_truncate_file (char *file, int lines);
 extern void add_history (char *string);
+extern int read_history (char *filename);
+extern int write_history (char *filename);
+extern void using_history (void);
+extern HIST_ENTRY *previous_history (void);
+extern int debugging_with_mdb;
+extern int mhtml_parser_interrupted;
+extern void mdb_reset_pushed_functions (void);
+
+extern char *tilde_expand (char *string);
 extern void mdb_initialize_readline (void);
 
 extern char **environ;
@@ -58,34 +80,81 @@ jmp_buf mdb_top_level_jmp_buffer;
 
 int mdb_loop_level = 0;
 void mdb_loop (void);
+static char *find_history_create (char *name);
+
+static int mdb_major = 2;
+static int mdb_minor = 4;
 
 int
 main (int argc, char *argv[])
 {
+  char *history_file = (char *)NULL;
+
   if (!*metahtml_copyright_string)
     abort ();
 
-  printf ("MDB 2.3: Interactive Debugger for Meta-HTMLv%s, by Brian J. Fox.",
-	  mhtml_version_string);
+  printf ("MDB %d.%d: Interactive Debugger for Meta-HTMLv%s, by Brian J. Fox.",
+	  mdb_major, mdb_minor, mhtml_version_string);
   printf ("\n%s\n", metahtml_copyright_string);
   printf ("Compiled for (%s)\n", MHTML_SYSTEM_TYPE);
 
   initialize_mdb (argc, argv);
   bootstrap_metahtml (1);
 
-  if (setjmp (mdb_top_level_jmp_buffer))
-    {
-      if (MDB_Interrupted)
-	printf ("Quit\n");
-      mdb_loop_level = 0;
-    }
+  history_file = find_history_create (".mdb_history");
+  history_truncate_file (history_file, 500);
+  read_history (history_file);
 
   while (!MDB_QuitFlag)
-    mdb_loop ();
+    {
+      /* signal (SIGINT, restart); */
+
+      if (setjmp (mdb_top_level_jmp_buffer))
+	{
+	  mhtml_parser_interrupted = 0;
+	  mdb_loop_level = 0;
+	  mdb_reset_pushed_functions ();
+	}
+
+      if (MDB_Interrupted)
+	{
+	  printf ("Quit\n");
+	  MDB_Interrupted = 0;
+	}
+
+      mdb_loop ();
+    }
+
+  /* Save the history. */
+  if (history_file != (char *)NULL)
+    write_history (history_file);
 
   return (0);
 }
 
+static char *
+find_history_create (char *name)
+{
+  static char full_history_path[2048];
+  char *use_name;
+
+  if (name == (char *)NULL)
+    name = strdup (".mdb_history");
+
+  if (*name != '/')
+    {
+      char *temp = (char *)xmalloc (10 + strlen (name));
+      sprintf (temp, "~/%s", name);
+      use_name = tilde_expand (temp);
+    }
+  else
+    use_name = strdup (name);
+
+  strcpy (full_history_path, use_name);
+  free (use_name);
+  return (full_history_path);
+}
+
 void
 mdb_loop (void)
 {
@@ -109,23 +178,42 @@ mdb_loop (void)
   prompt[i++] = ' ';
   prompt[i++] = '\0';
 
-  while (!done)
+  while (!done && !MDB_Interrupted)
     {
       char *result = (char *)NULL;
+      int add_to_hist = 1;
 
       line = readline (prompt);
 
       if (line == (char *)NULL)
-	line = strdup ("quit");
+	{
+	  line = strdup ("quit");
+	  add_to_hist = 0;
+	}
 
       printf ("\r");
       fflush (stdout);
 
       if (*line)
 	{
-	  result = mdb_command (line);
+	  if (add_to_hist)
+	    {
+	      HIST_ENTRY *entry;
 
-	  add_history (line);
+	      using_history ();
+
+	      entry = previous_history ();
+
+	      if ((entry == (HIST_ENTRY *)NULL) ||
+		  ((entry->line != (char *)NULL) &&
+		   ((entry->line[0] != line[0]) ||
+		    (strcmp (entry->line, line) != 0))))
+		{
+		  using_history ();
+		  add_history (line);
+		}
+	    }
+	  result = mdb_command (line);
 	}
       else
 	result = mdb_redo ();
@@ -140,7 +228,7 @@ mdb_loop (void)
 	  else
 	    result[i] = '\0';
 
-	  if (*result && !MDB_QuitFlag)
+	  if (*result && !MDB_QuitFlag && !MDB_Interrupted)
 	    printf ("%s\n", result);
 
 	  free (result);
@@ -148,7 +236,7 @@ mdb_loop (void)
 
       free (line);
 
-      if (MDB_QuitFlag || MDB_ContFlag)
+      if (MDB_QuitFlag || MDB_ContFlag || MDB_Interrupted)
 	break;
     }
 
@@ -158,11 +246,17 @@ mdb_loop (void)
   mdb_loop_level--;
 }
 
-static void
-restart (int sig)
+#define MDB_BROKEN_SIGINT_HANDLER 1
+
+void
+mdb_signal_restart (int sig)
 {
   MDB_Interrupted++;
+#if defined (MDB_BROKEN_SIGINT_HANDLER)
   mdb_throw_to_top_level ();
+#else
+  mhtml_parser_interrupted = 1;
+#endif
 }
 
 void
@@ -177,12 +271,15 @@ initialize_mdb (int argc, char *argv[])
   char working_dir[1024];
   char *result = getcwd (working_dir, 1023);
 
-  signal (SIGINT, restart);
+  debugging_with_mdb = 1;
+
+  signal (SIGINT, mdb_signal_restart);
 
   /* Generate PAGEFUNC_FUNCTION_PACKAGE by evaluating something. */
   {
     PAGE *page = page_create_page ();
-    bprintf (page, "<defsubst *mdb-version*>1.0</defsubst>\n");
+    bprintf (page, "<defsubst *mdb-version*>%d.%d</defsubst>\n",
+	     mdb_major, mdb_minor);
     page_process_page (page);
     if (page)
       page_free_page (page);
@@ -196,9 +293,32 @@ initialize_mdb (int argc, char *argv[])
   pagefunc_set_variable ("mhtml::http-to-host", "");
   pagefunc_set_variable ("mhtml::current-doc", "");
   pagefunc_set_variable ("mhtml::include-prefix", result);
-#if 0
   pagefunc_set_variable ("mhtml::relative-prefix", "");
-#endif
+
+  /* Try to get smart here.  If the include-prefix contains "/docs/...",
+     then the relative prefix is "/...", and the include-prefix goes up
+     to "/docs". */
+  if (result != (char *)NULL)
+    {
+      register int i;
+
+      for (i = 0; result[i] != '\0'; i++)
+	if (strncasecmp (result + i, "/docs", 5) == 0)
+	  {
+	    if ((result[i + 5] == '\0') || (result[i + 5] == '/'))
+	      {
+		char *incpref = (char *)xmalloc (1 + (i + 5));
+		strncpy (incpref, result, i + 5);
+		incpref[i + 5] = '\0';
+		pagefunc_set_variable ("mhtml::include-prefix", incpref);
+		free (incpref);
+		pagefunc_set_variable ("mhtml::relative-prefix",
+				       result + i +5);
+		break;
+	      }
+	  }
+    }
+
   pagefunc_set_variable ("mhtml::current-url", "");
   pagefunc_set_variable ("mhtml::current-url-sans-sid", "");
   pagefunc_set_variable ("mhtml::full-url", "");
@@ -210,6 +330,13 @@ initialize_mdb (int argc, char *argv[])
   pagefunc_set_variable ("mhtml::url-to-dir-sans-sid", "");
   pagefunc_set_variable ("mhtml::version", mhtml_version_string);
   pagefunc_set_variable ("mhtml::exec-path", (char *)getenv ("PATH"));
+  pagefunc_set_variable ("mdb::running-in-mdb", "true");
+
+      /* Set mhtml::require-directories[], to a reasonable value here. */
+      pagefunc_set_variable ("mhtml::require-directories[]",
+".\ntagsets\nmacros\nincludes\n\
+..\n../tagsets\n../macros\n../includes\n\
+../..\n../../tagsets\n../../macros\n../../includes");
 
   /* Get the environment into MDB. */
   {
@@ -319,3 +446,7 @@ initialize_mdb (int argc, char *argv[])
     }
 }
 
+
+#if defined (__cplusplus)
+}
+#endif
