@@ -43,6 +43,11 @@
 #include "mdb.h"
 #include "commands.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 #if !defined (__FUNCTION_DEF)
 #  define __FUNCTION_DEF
 typedef char **CPPFunction (char *, int, int);
@@ -424,3 +429,7 @@ symbol_completion_function (char *hint_text, int state)
   return ((char *)NULL);
 }
 
+
+#if defined (__cplusplus)
+}
+#endif
