@@ -26,12 +26,17 @@
 #endif
 
 
-extern char *xmalloc (int), *xrealloc (char *, int);
 #include <stdio.h>
 #include <stdlib.h>
 #include "rlconf.h"
 #include "readline.h"
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
+extern char *xmalloc (int), *xrealloc (char *, int);
 static int qsort_string_compare (char **s1, char **s2);
 
 FUNMAP **funmap = (FUNMAP **)NULL;
@@ -288,3 +293,7 @@ memory_error_and_abort ()
   abort ();
 }
 #endif /* STATIC_MALLOC */
+
+#if defined (__cplusplus)
+}
+#endif
