@@ -33,6 +33,10 @@
 #include "vi_keymap.c"
 #endif
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
 extern int rl_do_lowercase_version (int, int);
 extern int rl_rubout (int, int), rl_insert (int, int);
 
@@ -184,3 +188,7 @@ memory_error_and_abort (void)
   abort ();
 }
 #endif /* STATIC_MALLOC */
+
+#if defined (__cplusplus)
+}
+#endif
