@@ -31,18 +31,21 @@
 
 #include "language.h"
 
-static void pf_when (PFunArgs);
-#define COMPLEX_ELSE_IF 0
-
-#if defined (COMPLEX_ELSE_IF)
-static void pf_complex_if (PFunArgs);
+#if defined (__cplusplus)
+extern "C"
+{
 #endif
+static void pf_whenelse (PFunArgs);
+static void pf_complex_if (PFunArgs);
+
+/* #define COMPLEX_ELSE_IF */
 
 /* 2) Create a static table which associates function name, type, debug-flags,
       and address of code for each function. */
-static PFunDesc functab[] = {
+static PFunDesc ftab[] =
+{
   /*   tag	     complex? debug_level	   code    */
-  { "WHEN",		1,	 0,		pf_when },
+  { "WHENELSE",		1,	 0,		pf_whenelse },
 #if defined (COMPLEX_ELSE_IF)
   { "IF",		1,	 0,		pf_complex_if },
 #else
@@ -54,23 +57,48 @@ static PFunDesc functab[] = {
 void
 module_initialize (void)
 {
-  register int i;
-  Symbol *sym;
+  static int called = 0;
 
-  /* Install the names and pointers. */
-  for (i = 0; functab[i].tag != (char *)NULL; i++)
+  if (!called)
     {
-      sym = symbol_intern_in_package (mhtml_function_package, functab[i].tag);
-      sym->type = symtype_FUNCTION;
-      sym->values = (char **)(&functab[i]);
+      register int i;
+      Symbol *sym, *funcnames;
+
+      called++;
+      funcnames = symbol_intern ("modules::syms-of-elsewhen");
+
+      /* Install the names and pointers. */
+      for (i = 0; ftab[i].tag != (char *)NULL; i++)
+	{
+	  sym = symbol_intern_in_package (mhtml_function_package, ftab[i].tag);
+	  symbol_add_value (funcnames, ftab[i].tag);
+	  sym->type = symtype_FUNCTION;
+	  sym->values = (char **)(&ftab[i]);
+	}
     }
 }
 
 void _init (void) { module_initialize (); }
 
-/* Just like <when> ... </when> but allow <elsewhen> to appear in the body. */
-static void
-pf_when (PFunArgs)
+DOC_SECTION (ELSEWHEN-MODULE)
+DEFUNX (pf_elsewhen_module_synopsis, ,
+"This module is provided for those programmers who feel more comfortable with
+a balanced tag syntax for flow control over a balanced brace syntax.
+Using the tags in this package will greatly reduce the number of
+<funref flow-control group>, or <funref flow-control concat> tags, at the
+slight cost of interpreter speed.  In many cases, the clarity of the code
+is more important, so if you are one of those programmers that feel more
+comfortable with the tag-based syntax, you might consider the use of this
+module.")
+
+DEFMACRO (pf_whenelse, test,
+"Evaluate <var test>.  If the result is a non-empty string,
+then execute the <var body> statements.  If the tag &lt;elsewhen&gt;
+appears in the body, then the commands between that tag and the closing
+&lt;/whenelse&gt; will be executed if, and only if, the <var test> evaluates
+to the empty string.  This is a cleaner way to handle optional multiple
+statement execution rather than dealing with quoting everything inside
+of an <funref FLOW-CONTROL if> form.")
 {
   char *test = mhtml_evaluate_string (get_positional_arg (vars, 0));
   char *else_text = (char *)NULL;
@@ -84,7 +112,7 @@ pf_when (PFunArgs)
 
       /* Is it within our <when> body? */
       ours_p = page_indicator_owned_by
-	(body, "elsewhen", "when", &e_start, &e_end, else_start);
+	(body, "elsewhen", "whenelse", &e_start, &e_end, else_start);
 
       if (ours_p == 1)
 	{
@@ -115,7 +143,21 @@ pf_when (PFunArgs)
   xfree (test);
 }
 
-#if defined (COMPLEX_ELSE_IF)
+DEFMACROX (pf_cif, test &optional then-clause <else> else-clause,
+"First <var test> is evaluated. If the result is a non-whitespace only string,
+the statements between the <var then-clause> and the <var else-clause> are
+evaluated.  Otherwise, if the the <var else-clause> is present, it is
+evaluated. Although Meta-HTML has an <code>or</code> function, you can
+efficiently test for the presence of any of a group of variables with code
+similar to the following:
+<example>
+<if <get-var foo bar>>
+  \"Either FOO or BAR is present\"
+<else>
+  \"Neither FOO nor BAR is present\"
+</if>
+</example>")
+
 /* A non-lisp friendly <if> <else> </if> tag. */
 static void
 pf_complex_if (PFunArgs)
@@ -132,7 +174,7 @@ pf_complex_if (PFunArgs)
 
       /* Is it within our <when> body? */
       ours_p = page_indicator_owned_by
-	(body, "else", "if", &e_start, &e_end, else_start);
+	(body, "else", "cif", &e_start, &e_end, else_start);
 
       if (ours_p == 1)
 	{
@@ -162,4 +204,7 @@ pf_complex_if (PFunArgs)
 
   xfree (test);
 }
-#endif /* COMPLEX_ELSE_IF */
+
+#if defined (__cplusplus)
+}
+#endif
