@@ -41,6 +41,11 @@
 #  include <pwd.h>
 #endif
 
+#if defined (__cplusplus)
+extern "C"
+{
+#endif
+
 #if !defined (errno)
 extern int errno;
 #endif
@@ -936,7 +941,7 @@ static void
 loop_over_connections (void)
 {
   int sock = -1;
-  int sock_size = sizeof (struct sockaddr_in);
+  unsigned int sock_size = sizeof (struct sockaddr_in);
   struct in_addr host_address;
   struct sockaddr_in socket_address;
 #if defined (MHTTPD_HTML_ADMINISTRATION)  
@@ -1263,8 +1268,8 @@ loop_over_connections (void)
 			    {
 			      DBOBJ key, *val;
 
-			      key.data = (req->requester_addr);
-			      key.length = 1 + strlen (key.data);
+			      key.data =((unsigned char *)req->requester_addr);
+			      key.length = 1 + strlen ((char *)key.data);
 			      val = database_fetch (db, &key);
 
 			      if (val != (DBOBJ *)NULL)
@@ -1273,7 +1278,7 @@ loop_over_connections (void)
 				    {
 				      cache_found++;
 				      strncpy (dns_cached_result,
-					       val->data, val->length);
+					       (char *)val->data, val->length);
 				      dns_cached_result[val->length] = '\0';
 				      client_info_name = dns_cached_result;
 				      free (val->data);
@@ -1329,10 +1334,10 @@ loop_over_connections (void)
 		      if (db != (DBFILE)NULL)
 			{
 			  DBOBJ key, val;
-			  key.data = req->requester_addr;
-			  val.data = client_info_name;
-			  key.length = 1 + strlen (key.data);
-			  val.length = 1 + strlen (val.data);
+			  key.data = (unsigned char *)req->requester_addr;
+			  val.data = (unsigned char *)client_info_name;
+			  key.length = 1 + strlen ((char *)key.data);
+			  val.length = 1 + strlen ((char *)val.data);
 			  database_store (db, &key, &val);
 			  database_close (db);
 			}
@@ -1537,3 +1542,7 @@ loop_over_connections (void)
 	}
     }
 }
+
+#if defined (__cplusplus)
+}
+#endif
